SystemOrganization addCategory: #'PostgresV3-Core'!
SystemOrganization addCategory: #'PostgresV3-Core-Messages'!

!Date methodsFor: '*PostgresV3-Core' stamp: 'ul 9/11/2016 17:09'!
pg3PrintAsLiteralOn: stream

	self dayMonthYearDo: [ :day :month :year |
		stream 
			nextPut: $';
			print: year;
			nextPut: $-;
			print: month;
			nextPut: $-;
			print: day;
			nextPut: $' ]! !

!ByteArray methodsFor: '*PostgresV3-Core' stamp: 'ul 6/11/2017 22:28'!
pg3PrintAsLiteralOn: stream
	"Hex format. Requires PostgreSQL 9.0 or newer."

	| byte |
	stream nextPutAll: 'E''\\x'.
	1 to: self size do: [ :index |
		stream
			nextPut: ('0123456789ABCDEF' at: ((byte := self at: index) bitShift: -4) + 1);
			nextPut: ('0123456789ABCDEF' at: (byte bitAnd: 16rF) + 1) ].
	stream nextPut: $'! !

!Object methodsFor: '*PostgresV3-Core' stamp: 'ul 7/23/2010 14:25'!
asPG3SqlLiteral

	^String streamContents: [ :stream |
		self pg3PrintAsLiteralOn: stream ]! !

!Object methodsFor: '*PostgresV3-Core' stamp: 'ul 7/23/2010 14:25'!
pg3PrintAsLiteralOn: stream

	self printOn: stream! !

Object subclass: #PG3BinaryWriteStream
	instanceVariableNames: 'buffer position bufferSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3BinaryWriteStream commentStamp: 'ul 6/10/2017 22:45' prior: 0!
I am a write stream with a ByteArray buffer. My goal is efficient serialization of messages.

Instance Variables
	buffer:		<Object>
	bufferSize:		<Object>
	position:		<Object>

buffer
	- xxxxx

bufferSize
	- xxxxx

position
	- xxxxx
!

!PG3BinaryWriteStream class methodsFor: 'instance creation' stamp: 'ul 6/10/2017 23:14'!
new

	self error: 'Use #new: to create an instance'! !

!PG3BinaryWriteStream class methodsFor: 'instance creation' stamp: 'ul 6/10/2017 23:13'!
new: bufferSize

	^self basicNew initialize: bufferSize! !

!PG3BinaryWriteStream methodsFor: 'accessing' stamp: 'ul 6/10/2017 22:58'!
contents

	^buffer copyFrom: 1 to: position! !

!PG3BinaryWriteStream methodsFor: 'private' stamp: 'ul 6/10/2017 22:54'!
grow

	self growTo: bufferSize * 2 "assumes than bufferSize > 0"! !

!PG3BinaryWriteStream methodsFor: 'private' stamp: 'ul 6/10/2017 22:55'!
growTo: newBufferSize
	"Replace buffer with another ByteArray of size newBufferSize. Assume that newBufferSize is greater than bufferSize."

	| newBuffer |
	newBuffer := ByteArray new: newBufferSize.
	newBuffer
		replaceFrom: 1
		to: position
		with: buffer
		startingAt: 1.
	buffer := newBuffer.
	bufferSize := newBufferSize! !

!PG3BinaryWriteStream methodsFor: 'initialize-release' stamp: 'ul 6/10/2017 22:46'!
initialize: newBufferSize

	buffer := ByteArray new: newBufferSize.
	position := 0.
	bufferSize := newBufferSize! !

!PG3BinaryWriteStream methodsFor: 'accessing' stamp: 'ul 6/10/2017 22:48'!
nextPut: aByte

	position = bufferSize ifTrue: [ self grow ].
	 ^buffer at: (position := position + 1) put: aByte! !

!PG3BinaryWriteStream methodsFor: 'accessing' stamp: 'ul 6/10/2017 23:01'!
nextPutAll: aByteArray

	| size newPosition |
	size := aByteArray size.
	(newPosition := position + size) > bufferSize ifTrue: [
		self growTo: newPosition asLargerPowerOfTwo ].
	buffer
		replaceFrom: position + 1
		to: newPosition
		with: aByteArray
		startingAt: 1.
	position := newPosition.
	^aByteArray! !

!PG3BinaryWriteStream methodsFor: 'accessing' stamp: 'ul 6/10/2017 23:01'!
originalContents

	^buffer! !

!PG3BinaryWriteStream methodsFor: 'accessing' stamp: 'ul 6/10/2017 23:01'!
position

	^position! !

!PG3BinaryWriteStream methodsFor: 'accessing' stamp: 'ul 6/10/2017 23:23'!
position: anInteger

	position := anInteger! !

!PG3BinaryWriteStream methodsFor: 'accessing' stamp: 'ul 6/10/2017 23:15'!
reset

	position := 0! !

!PG3BinaryWriteStream methodsFor: 'as yet unclassified' stamp: 'ul 6/13/2017 23:33'!
sendDataTo: aSocket

	position = 0 ifTrue: [ ^self ].
	aSocket sendData: buffer count: position.
	position := 0! !

!PG3BinaryWriteStream methodsFor: 'accessing' stamp: 'ul 6/11/2017 18:14'!
writeInt16: anInteger

	| newPosition |
	(newPosition := position + 2) > bufferSize ifTrue: [
		self growTo: newPosition asLargerPowerOfTwo ].
	buffer shortAt: position + 1 put: anInteger bigEndian: true.
	position := newPosition.
	^anInteger
	! !

!PG3BinaryWriteStream methodsFor: 'accessing' stamp: 'ul 6/10/2017 23:10'!
writeInt32: anInteger

	| newPosition |
	(newPosition := position + 4) > bufferSize ifTrue: [
		self growTo: newPosition asLargerPowerOfTwo ].
	buffer longAt: position + 1 put: anInteger bigEndian: true.
	position := newPosition.
	^anInteger
	! !

!PG3BinaryWriteStream methodsFor: 'writing' stamp: 'ul 6/13/2017 21:14'!
writeObject: anObject formatCode: formatCode

	formatCode = 0 ifTrue: [
		| startPosition text |
		startPosition := position.
		position := position + 4.
		text := String streamContents: [ :stream | anObject pg3PrintAsLiteralOn: stream ].
		self nextPutAll: text.
		buffer longAt: startPosition + 1 put: text size bigEndian: true.
		^self ].
	self halt! !

!PG3BinaryWriteStream methodsFor: 'accessing' stamp: 'ul 6/10/2017 23:19'!
writeString: aByteString

	| size newPosition |
	size := aByteString size + 1.
	(newPosition := position + size) > bufferSize ifTrue: [
		self growTo: newPosition asLargerPowerOfTwo ].
	buffer
		replaceFrom: position + 1
		to: newPosition - 1 "Ignore the terminating zero"
		with: aByteString
		startingAt: 1.
	buffer at: (position := newPosition) put: 0. "Terminate the string with a 0 byte."
	^aByteString! !

Object subclass: #PG3ColumnDescription
	instanceVariableNames: 'name tableId columnId typeId typeSize typeModifier formatCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

PG3ColumnDescription class
	instanceVariableNames: 'converterArray'!

PG3ColumnDescription class
	instanceVariableNames: 'converterArray'!

!PG3ColumnDescription class methodsFor: 'accessing' stamp: 'ul 3/22/2009 21:32'!
converterArray

	^converterArray! !

!PG3ColumnDescription class methodsFor: 'class initialization' stamp: 'ul 6/11/2017 22:36'!
initialize

	self initializeConverterArray


	 
! !

!PG3ColumnDescription class methodsFor: 'class initialization' stamp: 'ul 9/21/2009 00:41'!
initializeConverterArray

	| textConverterSelectors |
	textConverterSelectors := self textConverterSelectors.
	converterArray := Array 
		new: (textConverterSelectors detectMax: [ :each | each first ]) first + 1 " make sure that the last element is the default element (#next:), because we will access the elements with #atOrLast:"
		withAll: #next:.
	textConverterSelectors do: [ :each |
		converterArray at: each first put: each second ]
	 
! !

!PG3ColumnDescription class methodsFor: 'accessing' stamp: 'klub 3/25/2015 14:08'!
textConverterSelectors

	^#(
			(16 #parseBooleanLength:) "bool"
			(17 #parseByteArrayLength:) "bytea"
			(18 #parseCharacterLength:) "char"
			(19 #parseStringLength:) "name"
			(20 #parseIntegerLength:) "int8"
			(21 #parseIntegerLength:) "int2"
			(22 #parseIntegerVectorLength:) "int2vector"
			(23 #parseIntegerLength:) "int4"	
			(24 #parseStringLength:) "regproc"
			(25 #parseStringLength:) "text"
			(26 #parseIntegerLength:) "oid"
			(114 #parseJsonLength:) "json"
			(142 #parseXmlLength:) "xml"
			(143 #parseXmlArrayLength:) "xml[]"
			(199 #parseJsonArrayLength:) "json[]"
			(1043 #parseStringLength:) "character varying"		
			(1042 #parseStringLength:) "char"		
			
			(1082 #parseDateLength:) "date"		
		
			(1114 #parseTimestampWithoutTimezoneLength:) "timestamp"		
			(1184 #parseTimestampWithTimezoneLength:) "timestamptz"

			(1700 #parseScaledDecimalLength:) "numeric"
		
			(700 #parseFloatLength:) "float4"
			(701 #parseFloatLength:) "float8"
				
			(705 #parseStringLength:) "unknown"
		
			(1000 #parseBooleanArrayLength:) "bool[]"
			(1003 #parseStringArrayLength:) "name[]"
			(1005 #parseIntegerArrayLength:) "int2[]"
			(1009 #parseStringArrayLength:) "text[]"
			(1007 #parseIntegerArrayLength:) "int4[]"
			(1014 #parseStringArrayLength:) "character[]"
			(1015 #parseStringArrayLength:) "character varying[]"
			(1016 #parseIntegerArrayLength:) "int2[]"
			(1021 #parseFloatArrayLength:) "float4[]"
			(1022 #parseFloatArrayLength:) "float8[]"
			(1083 #parseTimeLength:) "time"
			(1183 #parseTimeArrayLength:) "_time"
			(1231 #parseScaledDecimalArrayLength:) "numeric[]"
		
			(1790 #parseStringLength:) "refcursor[]"
			(2278 #parseVoidLength:)
		)! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
columnId
	^ columnId! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
columnId: anObject
	columnId := anObject! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 6/15/2017 23:49'!
converterSelector

	formatCode = 0 ifTrue: [ "text format"
		^self class converterArray atOrLast: typeId ].
	formatCode = 1 ifFalse: [ "unkonwn format"
		^#next: ].
	"binary format"
	typeId
		caseOf: {
			[ 16 ] -> [ ^#next: ]. "TODO boolean" 
			[ 17 ] -> [ ^#next: ]. "bytea" 
			[ 18 ] -> [ ^#next: ]. "TODO char"
			[ 19 ] -> [ ^#parseBinaryStringLength: ]. "name"
			[ 20 ] -> [ ^#parseBinaryLong64OfLength: ]. "int8"
			[ 21 ] -> [ ^#parseBinaryShortOfLength: ]. "int2"
			[ 23 ] -> [ ^#parseBinaryLongOfLength: ]. "int4"
			[ 25 ] -> [ ^#parseBinaryStringLength: ]. "text"
			[ 26 ] -> [ ^#parseBinaryLongOfLength: ]. "oid"
			[ 1042 ] -> [ ^#parseBinaryStringLength: ]. "char"		
			[ 700 ] -> [ ^#parseBinaryFloat32OfLength: ]. "float4"
			[ 701 ] -> [ ^#parseBinaryFloat64OfLength: ]. "float8"
			[ 1043 ] -> [ ^#parseBinaryStringLength: ]. "character varying"
		}
		otherwise: [ ^#next: ].
! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
formatCode
	^ formatCode! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
formatCode: anObject
	formatCode := anObject! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
name
	^ name! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
name: anObject
	name := anObject! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 3/7/2009 07:08'!
printOn: aStream

	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: name;
		nextPutAll: ', ';
		print: typeId;
		nextPut: $)! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
tableId
	^ tableId! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
tableId: anObject
	tableId := anObject! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
typeId
	^ typeId! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
typeId: anObject
	typeId := anObject! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
typeModifier
	^ typeModifier! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
typeModifier: anObject
	typeModifier := anObject! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
typeSize
	^ typeSize! !

!PG3ColumnDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:17'!
typeSize: anObject
	typeSize := anObject! !

Object subclass: #PG3Connection
	instanceVariableNames: 'socket arguments socketReadStream sendBufferStream state processId secretKey parameters states resultSets transactionState currentConverterSelectors textConverter noticeHandler commandCompleteHandler currentRowClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3Connection methodsFor: 'startup-protocol' stamp: 'ul 2/16/2009 05:18'!
addParameterStatus: aPG3ParameterStatus 
	
	parameters at: aPG3ParameterStatus name put: aPG3ParameterStatus value! !

!PG3Connection methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:32'!
arguments

	^arguments! !

!PG3Connection methodsFor: 'accessing' stamp: 'ul 2/15/2009 00:39'!
arguments: aPG3ConnectionArguments

	arguments := aPG3ConnectionArguments! !

!PG3Connection methodsFor: 'extended query protocol' stamp: 'ul 6/13/2017 21:07'!
bind: queryName portal: portalName parameters: anArray
	"Execute the query given in aString with the simple query protocol"

	| resultSetsToReturn |
	socketReadStream resetOffset.
	resultSets := OrderedCollection new: 1.
	self
		queueMessage: (PG3Bind new
			portalName: portalName
			preparedStatementName: queryName
			parameterFormatCodes: #[0]
			parameters: anArray
			resultFormatCodes: #[0];
			yourself);
		queueMessage: PG3Sync new;
		flushMessages.
	state := states at: #Binding.
	self processStates.
	resultSetsToReturn := resultSets.
	resultSets := nil.
	^resultSetsToReturn! !

!PG3Connection methodsFor: 'extended query protocol' stamp: 'ul 6/14/2017 10:31'!
bindAndExecute: queryName portal: portalName parameters: anArray maxRows: maxRows
	"Execute the query given in aString with the simple query protocol"

	| resultSetsToReturn |
	socketReadStream resetOffset.
	resultSets := OrderedCollection new: 1.
	self
		queueMessage: (PG3Bind new
			portalName: portalName
			preparedStatementName: queryName
			parameterFormatCodes: #[0]
			parameters: anArray
			resultFormatCodes: #[1];
			yourself);
		queueMessage: (PG3Describe new
			portalNamed: portalName;
			yourself);
		queueMessage: (PG3Execute new
			executePortal: portalName
			maxRows: maxRows;
			yourself);
		queueMessage: PG3Sync new;
		flushMessages.
	state := states at: #Binding.
	self processStates.
	resultSetsToReturn := resultSets.
	resultSets := nil.
	^resultSetsToReturn! !

!PG3Connection methodsFor: 'extended query protocol' stamp: 'ul 6/13/2017 21:15'!
bindComplete: aPG3BindComplete
	"Do nothing"! !

!PG3Connection methodsFor: 'as yet unclassified' stamp: 'ul 6/11/2017 22:06'!
cancelRequest
	"Attempt to create another connection to the server and cancel the request this connection has issued. This method will ignore any connection pool present."

	self class new
		arguments: arguments;
		ensureConnection;
		sendMessage: (PG3CancelRequest for: self)! !

!PG3Connection methodsFor: 'simple-query-protocol' stamp: 'klub 8/11/2011 15:02'!
commandComplete: aPG3CommandCompleteMessage

	commandCompleteHandler ifNotNil: [
		commandCompleteHandler value: aPG3CommandCompleteMessage ]! !

!PG3Connection methodsFor: 'accessing' stamp: 'klub 8/11/2011 15:03'!
commandCompleteHandler: aBlockOrNil

	commandCompleteHandler := aBlockOrNil! !

!PG3Connection methodsFor: 'accessing' stamp: 'ul 2/26/2009 11:55'!
currentConverterSelectors

	^currentConverterSelectors! !

!PG3Connection methodsFor: 'accessing' stamp: 'klub 5/21/2014 16:08'!
currentRowClass

	^currentRowClass ifNil: [ PG3Row ]! !

!PG3Connection methodsFor: 'simple-query-protocol' stamp: 'ul 9/5/2009 21:40'!
dataRow: aPG3DataRow

	resultSets last rows add: aPG3DataRow values! !

!PG3Connection methodsFor: 'transaction helpers' stamp: 'kb 8/4/2010 20:50'!
deferAllConstraints

	self execute: 'set constraints all deferred'! !

!PG3Connection methodsFor: 'as yet unclassified' stamp: 'klub 9/10/2016 02:33'!
ensureConnection

	self initializeSocket.
	socket 
		connectTo: (arguments hostAddress ifNil: [ 
			NetNameResolver
				addressForName: arguments hostname
				timeout: 20 ])
		port: arguments port
		waitForConnectionFor: arguments connectTimeout! !

!PG3Connection methodsFor: 'simple-query-protocol' stamp: 'klub 5/21/2014 16:14'!
execute: aString
	"Execute the query given in aString with the simple query protocol"

	^self execute: aString rowClass: nil! !

!PG3Connection methodsFor: 'simple-query-protocol' stamp: 'klub 5/21/2014 16:39'!
execute: aString rowClass: aClass
	"Execute the query given in aString with the simple query protocol"

	| returnedResultSets |
	socketReadStream resetOffset.
	currentRowClass := aClass.
	self sendMessage: (PG3Query new
		queryString: (textConverter convertFromSqueak: aString);
		yourself).
	resultSets := OrderedCollection new: 3.
	state := states at: #Querying.
	self processStates.
	returnedResultSets := resultSets.
	resultSets := nil. "We no longer need them here, and we don't want to hold a reference to them."
	currentRowClass := nil.
	^returnedResultSets! !

!PG3Connection methodsFor: 'extended query protocol' stamp: 'ul 6/13/2017 23:12'!
executePortal: portalName maxRows: maxRows
	
	| resultSetsToReturn |
	socketReadStream resetOffset.
	resultSets := OrderedCollection new: 1.
	self
		queueMessage: (PG3Execute new
			executePortal: portalName
			maxRows: maxRows;
			yourself);
		queueMessage: PG3Sync new;
		flushMessages.
	state := states at: #Executing.
	self processStates.
	resultSetsToReturn := resultSets.
	resultSets := nil.
	^resultSetsToReturn! !

!PG3Connection methodsFor: 'as yet unclassified' stamp: 'ul 6/13/2017 23:33'!
flushMessages

	sendBufferStream sendDataTo: socket! !

!PG3Connection methodsFor: 'as yet unclassified' stamp: 'ul 3/6/2009 04:06'!
handleError: aPG3ErrorResponse

	resultSets add: aPG3ErrorResponse! !

!PG3Connection methodsFor: 'as yet unclassified' stamp: 'ul 2/15/2009 20:26'!
initializeSocket

	socket ifNotNil: [ socket destroy ].
	socket := Socket tcpCreateIfFail: [ self error: 'Could not create socket!!' ].
	socketReadStream := PG3SocketReadStream on: socket! !

!PG3Connection methodsFor: 'testing' stamp: 'ul 8/30/2009 21:19'!
isConnected

	^socket isConnected! !

!PG3Connection methodsFor: 'testing' stamp: 'ul 3/6/2009 04:00'!
isIdle

	^transactionState = 73 "$I asciiValue"! !

!PG3Connection methodsFor: 'testing' stamp: 'ul 3/6/2009 04:01'!
isInFailedTransaction

	^transactionState = 69 "$E asciiValue"! !

!PG3Connection methodsFor: 'testing' stamp: 'ul 3/6/2009 04:01'!
isInTransaction

	^transactionState = 84 "$T asciiValue"! !

!PG3Connection methodsFor: 'testing' stamp: 'ul 9/5/2009 16:47'!
isQuerying

	^state
		ifNil: [ false ]
		ifNotNil: [
			state name = #Querying ]
		! !

!PG3Connection methodsFor: 'testing' stamp: 'ul 8/30/2009 21:25'!
isReadyForQuery

	^state
		ifNil: [ false ]
		ifNotNil: [
			state name = #WaitingForQuery ]
		! !

!PG3Connection methodsFor: 'accessing' stamp: 'klub 8/11/2011 15:03'!
noticeHandler: aBlockOrNil

	noticeHandler := aBlockOrNil! !

!PG3Connection methodsFor: 'extended query protocol' stamp: 'ul 6/14/2017 10:27'!
parse: queryString named: name parameters: anArray
	"Execute the query given in aString with the simple query protocol"

	| resultSetsToReturn |
	socketReadStream resetOffset.
	resultSets := OrderedCollection new: 1.
	self
		queueMessage: (PG3Parse new
			preparedStatementName: (textConverter convertFromSqueak: name)
				queryString: (textConverter convertFromSqueak: queryString)
				parameterTypes: anArray
			yourself);
		"queueMessage: (PG3Describe new
			statementNamed: (textConverter convertFromSqueak: name);
			yourself);"
		queueMessage: PG3Sync new;
		flushMessages.
	state := states at: #Parsing.
	self processStates.
	resultSetsToReturn := resultSets.
	resultSets := nil.
	^resultSetsToReturn! !

!PG3Connection methodsFor: 'extended query protocol' stamp: 'ul 6/12/2017 20:45'!
parseComplete: message
	"nothing to do for now. normally register the prepared statement if it's named"! !

!PG3Connection methodsFor: 'accessing' stamp: 'ul 8/30/2009 20:58'!
processId

	^processId! !

!PG3Connection methodsFor: 'as yet unclassified' stamp: 'ul 2/26/2009 15:43'!
processStates
	
	[ state isStopState ] whileFalse: [
		| message |
		message := self receiveMessage.
		state := state receive: message from: self ]! !

!PG3Connection methodsFor: 'as yet unclassified' stamp: 'ul 6/16/2017 19:09'!
queueMessage: aPG3Message

	sendBufferStream
		ifNil: [ sendBufferStream := PG3BinaryWriteStream new: 1024 ]
		ifNotNil: [ 
			"Smoothly migrate the WriteStream to PG3BinaryWriteStream."
			sendBufferStream class == PG3BinaryWriteStream
				ifFalse: [ sendBufferStream := PG3BinaryWriteStream new: 1024 ] ].
	aPG3Message writeOn: sendBufferStream! !

!PG3Connection methodsFor: 'simple-query-protocol' stamp: 'ul 2/17/2009 03:51'!
readyForQuery: aPG3ReadyForQuery

	transactionState := aPG3ReadyForQuery transactionState

	! !

!PG3Connection methodsFor: 'as yet unclassified' stamp: 'ul 11/20/2013 12:21'!
receiveMessage

	^PG3TypedMessage dispatchFrom: socketReadStream connection: self! !

!PG3Connection methodsFor: 'simple-query-protocol' stamp: 'klub 8/11/2011 14:45'!
receiveNotice: aPG3NoticeResponse

	noticeHandler ifNotNil: [ noticeHandler value: aPG3NoticeResponse ]! !

!PG3Connection methodsFor: 'simple-query-protocol' stamp: 'ul 9/4/2009 17:58'!
receiveNotification: aPG3NotificationResponse

	"do nothing"! !

!PG3Connection methodsFor: 'startup-protocol' stamp: 'ul 8/30/2009 20:58'!
registerBackendKeyData: aPG3BackendKeyData

	processId := aPG3BackendKeyData processId.
	secretKey := aPG3BackendKeyData secretKey! !

!PG3Connection methodsFor: 'transaction helpers' stamp: 'kb 8/4/2010 20:50'!
releaseSavepoint: aString

	self execute: 'release ', aString asPG3SqlIdentifier! !

!PG3Connection methodsFor: 'startup-protocol' stamp: 'ul 6/11/2017 21:11'!
respondToAuthenticationClearTextRequest: message

	self sendMessage: arguments clearTextAuthenticationMessage! !

!PG3Connection methodsFor: 'startup-protocol' stamp: 'ul 6/11/2017 22:03'!
respondToAuthenticationCryptPasswordRequest: message

	self sendMessage: (arguments cryptAuthenticationMessageWithSalt: message salt)! !

!PG3Connection methodsFor: 'startup-protocol' stamp: 'ul 6/11/2017 21:09'!
respondToAuthenticationMD5PasswordRequest: message

	self sendMessage: (arguments md5AuthenticationMessageWithSalt: message salt)! !

!PG3Connection methodsFor: 'transaction helpers' stamp: 'kb 8/4/2010 20:50'!
rollback

	self execute: 'rollback'! !

!PG3Connection methodsFor: 'transaction helpers' stamp: 'kb 8/4/2010 20:50'!
rollbackTo: aString

	self execute: 'rollback to ', aString asPG3SqlIdentifier! !

!PG3Connection methodsFor: 'simple-query-protocol' stamp: 'ul 2/26/2009 11:57'!
rowDescription: aPG3RowDescription 

	resultSets add: (PG3ResultSet new
		rowDescription: aPG3RowDescription;
		rows: OrderedCollection new;
		yourself).
	currentConverterSelectors := aPG3RowDescription converterSelectors! !

!PG3Connection methodsFor: 'transaction helpers' stamp: 'kb 8/4/2010 20:51'!
savepoint: aString

	self execute: 'savepoint ', aString asPG3SqlIdentifier! !

!PG3Connection methodsFor: 'accessing' stamp: 'ul 8/30/2009 20:58'!
secretKey

	^secretKey! !

!PG3Connection methodsFor: 'as yet unclassified' stamp: 'ul 6/13/2017 23:34'!
sendMessage: aPG3Message

	self
		queueMessage: aPG3Message;
		flushMessages
				! !

!PG3Connection methodsFor: 'transaction helpers' stamp: 'kb 8/4/2010 20:51'!
show: aString

	^self execute: 'show ', aString asPG3SqlIdentifier! !

!PG3Connection methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:38'!
socket

	^socket! !

!PG3Connection methodsFor: 'accessing' stamp: 'ul 2/15/2009 20:56'!
socketReadStream

	^socketReadStream! !

!PG3Connection methodsFor: 'startup-protocol' stamp: 'ul 9/16/2009 20:20'!
startup

	self ensureConnection.
	self sendMessage: (PG3StartupMessage for: self).
	states := PG3ServerState createStateGraph.
	state := states at: #Authenticating.
	parameters := Dictionary new.
	self processStates.
	arguments automaticStringConversion
		ifFalse: [ self textConverter: PG3NullTextConverter new ]
		ifTrue: [ 
			self textConverter:(PG3SqueakTextConverter 
				for: (parameters 
					at: 'server_encoding'
					ifAbsent: [ 'UTF8' ])) ]! !

!PG3Connection methodsFor: 'accessing' stamp: 'ul 2/16/2009 05:01'!
state

	^state! !

!PG3Connection methodsFor: 'accessing' stamp: 'ul 2/16/2009 05:02'!
state: aPG3ServerState

	state := aPG3ServerState! !

!PG3Connection methodsFor: 'as yet unclassified' stamp: 'ul 2/17/2009 06:12'!
terminate

	socket ifNil: [ ^self ].
	socket isConnected ifTrue: [
		self sendMessage: PG3Terminate new.
		socket close ]! !

!PG3Connection methodsFor: 'accessing' stamp: 'kb 9/15/2009 17:36'!
textConverter: aPG3TextConverter

	textConverter := aPG3TextConverter.
	socketReadStream textConverter: aPG3TextConverter.! !

PG3Connection subclass: #PG3ListenerConnection
	instanceVariableNames: 'listenerProcess actions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3ListenerConnection methodsFor: 'listen-notify-subprotocol' stamp: 'ul 9/4/2009 18:10'!
createListenerProcess

	listenerProcess ifNotNil: [ 
		^self ].
	listenerProcess := [ self listenerProcess ] newProcess.
	listenerProcess resume! !

!PG3ListenerConnection methodsFor: 'simple-query-protocol' stamp: 'ul 9/4/2009 18:32'!
execute: aString

	self error: 'This connection can not execute queries'! !

!PG3ListenerConnection methodsFor: 'listen-notify-subprotocol' stamp: 'ul 9/4/2009 18:08'!
killListenerProcess 

	listenerProcess ifNotNil: [
		listenerProcess terminate.
		listenerProcess := nil ]! !

!PG3ListenerConnection methodsFor: 'listen-notify-subprotocol' stamp: 'kb 7/23/2010 02:04'!
listen: aString do: aValueable
	
	self killListenerProcess.
	super execute: 'listen ', aString asPG3SqlIdentifier.
	actions at: aString put: aValueable.
	self createListenerProcess! !

!PG3ListenerConnection methodsFor: 'listen-notify-subprotocol' stamp: 'ul 9/4/2009 18:15'!
listenerProcess

	[ 
		| message |
		message := self receiveMessage.
		state := state receive: message from: self ] repeat! !

!PG3ListenerConnection methodsFor: 'listen-notify-subprotocol' stamp: 'klub 6/2/2014 17:06'!
receiveNotification: aPG3NotificationResponse

	(actions at: aPG3NotificationResponse conditionName ifAbsent: [ ^self ])
		cull: aPG3NotificationResponse payload! !

!PG3ListenerConnection methodsFor: 'startup-protocol' stamp: 'ul 9/4/2009 17:54'!
startup 

	actions := Dictionary new.
	super startup
	! !

!PG3ListenerConnection methodsFor: 'as yet unclassified' stamp: 'ul 9/4/2009 20:50'!
terminate

	self killListenerProcess.
	super terminate! !

!PG3ListenerConnection methodsFor: 'listen-notify-subprotocol' stamp: 'ul 9/4/2009 18:26'!
unlisten: aString

	(actions includesKey: aString) ifFalse: [
		^self ].
	self killListenerProcess.
	super execute: 'unlisten ', aString.
	actions removeKey: aString.
	actions ifNotEmpty: [ self createListenerProcess ]! !

Object subclass: #PG3ConnectionArguments
	instanceVariableNames: 'username password databaseName hostAddress hostname port connectTimeout automaticStringConversion'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3ConnectionArguments commentStamp: 'ul 5/14/2010 19:04' prior: 0!
I hold the arguments of a connection which are used when the connection is being established.

Instance Variables
	automaticStringConversion:		<Boolean>
	connectTimeout:		<Integer>
	databaseName:		<String>
	hostname:		<String>
	password:		<String>
	port:		<Integer>
	username:		<String>

automaticStringConversion
	- if true, then automatically create and install a TextConverter object based on the 'server_encoding' parameter sent by the server, otherwise don't perform character encoding/decoding. Default is true.

connectTimeout
	- the number of milliseconds to wait for a connection to be created. Default 5000 (5 seconds).

databaseName
	- the name of the database to connect to

hostname
	- the name of the host to connect to

password
	- the password of the account to be used

port
	- the port of the server to connect to. Default 5432.

username
	- the username of the account to be used
	
	
Usage examples:
===========

Setting all arguments:
arguments := PG3ConnectionArguments new
	hostname: 'localhost';
	databaseName: 'db2;
	username: 'user';
	password: 'secret';
	automaticStringConversion: false;
	connectTimeout: 1000;
	yourself
	
The easiest way to create a connection setting only the necessary arguments:
connection := PG3ConnectionArguments new
	hostname: 'localhost';
	databaseName: 'db2;
	username: 'squeaker';
	password: 'secret';
	newConnection.
	!

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'kb 9/15/2009 17:41'!
automaticStringConversion

	^automaticStringConversion ifNil: [ true ]! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:29'!
automaticStringConversion: aBoolean

	automaticStringConversion := aBoolean! !

!PG3ConnectionArguments methodsFor: 'authentication' stamp: 'ul 6/11/2017 21:50'!
clearTextAuthenticationMessage

	^PG3PasswordMessage new
		password: (password convertToEncoding: 'utf-8');
		yourself! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 2/15/2009 00:41'!
connectTimeout
	
	^connectTimeout ifNil: [ 5000 ]! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 2/15/2009 00:41'!
connectTimeout: milliseconds 
	
	connectTimeout := milliseconds! !

!PG3ConnectionArguments methodsFor: 'authentication' stamp: 'ul 6/11/2017 22:25'!
cryptAuthenticationMessageWithSalt: salt

	| des key encodedPassword |
	des := Smalltalk classNamed: #DES.
	des ifNil: [ self error: 'This authentication method requires DES. Please install the Cryptography package.' ].
	key := ByteArray new: 8.
	encodedPassword := password convertToEncoding: 'utf-8'.
	1 to: (key size min: encodedPassword size) do: [ :index |
		key at: index put: (((encodedPassword basicAt: index) bitShift: 1) bitAnd: 16rFF) ].
	des new
		key: key;
		encryptBlock: key.
	self error: 'TODO'. "Mangle key based on salt."
	^PG3PasswordMessage new
		password: salt, key asString;
		yourself! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:32'!
databaseName

	^databaseName! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:32'!
databaseName: aString

	databaseName := aString! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'klub 9/10/2016 02:32'!
hostAddress

	^hostAddress! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'klub 9/10/2016 02:32'!
hostAddress: aByteArray

	hostAddress := aByteArray! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:32'!
hostname

	^hostname! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:32'!
hostname: aString

	hostname := aString! !

!PG3ConnectionArguments methodsFor: 'authentication' stamp: 'ul 6/11/2017 22:25'!
md5AuthenticationMessageWithSalt: salt

	| md5Hasher |
	md5Hasher := self md5Hasher.
	^PG3PasswordMessage new
		password: 'md5', (
			md5Hasher value: 
				(md5Hasher value: (password, username convertToEncoding: 'utf-8')), 
					salt asString);
		yourself! !

!PG3ConnectionArguments methodsFor: 'utility' stamp: 'ul 6/11/2017 21:53'!
md5Hasher
	"Return a block with converts its argument to a lowercase hexadecimal MD5 hash text."

	(Smalltalk classNamed: #WebUtils) ifNotNil: [ :webUtils |
		^[ :aByteString | webUtils md5Digest: aByteString ] ].
	(Smalltalk classNamed: #MD5) ifNotNil: [ :md5 |
		^[ :aByteString | (md5 hashMessage: aByteString) hex asLowercase ] ].
	self error: 'No suitable MD5 implementation found. Please install Cryptography or WebClient.'! !

!PG3ConnectionArguments methodsFor: 'connection creation' stamp: 'ul 5/14/2010 16:29'!
newConnection
	"Create and return a new connection with these arguments."

	^PG3Connection new
		arguments: self;
		yourself! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:32'!
password

	^password! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:33'!
password: aString

	password := aString! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:33'!
port

	^port! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:33'!
port: anInteger

	port := anInteger! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:33'!
username

	^username! !

!PG3ConnectionArguments methodsFor: 'accessing' stamp: 'ul 5/14/2010 16:34'!
username: aString

	username := aString! !

Object subclass: #PG3Message
	instanceVariableNames: 'connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

PG3Message subclass: #PG3CancelRequest
	instanceVariableNames: 'processId secretKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3CancelRequest class methodsFor: 'instance creation' stamp: 'ul 8/30/2009 18:20'!
for: aPG3Connection

	^self new
		processId: aPG3Connection processId;
		secretKey: aPG3Connection secretKey;
		yourself! !

!PG3CancelRequest methodsFor: 'accessing' stamp: 'ul 8/30/2009 18:12'!
processId: anInteger

	processId := anInteger! !

!PG3CancelRequest methodsFor: 'accessing' stamp: 'ul 8/30/2009 18:12'!
secretKey: anInteger

	secretKey := anInteger! !

!PG3CancelRequest methodsFor: 'writing' stamp: 'ul 6/10/2017 23:22'!
writeDataOn: aStream

	aStream
		writeInt32: 80877102; "1234 << 16 + 5678"
		writeInt32: processId;
		writeInt32: secretKey
		
		

	! !

!PG3Message methodsFor: 'accessing' stamp: 'ul 2/15/2009 18:09'!
connection

	^connection! !

!PG3Message methodsFor: 'accessing' stamp: 'ul 2/15/2009 18:09'!
connection: aPG3Connection

	connection := aPG3Connection! !

!PG3Message methodsFor: 'writing' stamp: 'ul 2/15/2009 00:00'!
writeDataOn: aStream! !

!PG3Message methodsFor: 'writing' stamp: 'ul 6/10/2017 23:23'!
writeOn: aStream
	"Leave 4 bytes for the length, then write the message, and finally fill the length."

	| lengthPosition endPosition |
	lengthPosition := aStream position.
	aStream position: lengthPosition + 4.
	self writeDataOn: aStream.
	endPosition := aStream position.
	aStream 
		position: lengthPosition;
		writeInt32: endPosition - lengthPosition;
		position: endPosition
	! !

PG3Message subclass: #PG3SSLRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

PG3Message subclass: #PG3StartupMessage
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3StartupMessage class methodsFor: 'instance creation' stamp: 'ul 2/15/2009 22:34'!
for: connection

	| arguments |
	arguments := connection arguments.
	^self new
		addParameter: 'user' value: arguments username;
		addParameter: 'database' value: arguments databaseName;
		yourself! !

!PG3StartupMessage methodsFor: 'adding' stamp: 'ul 2/14/2009 23:51'!
addParameter: parameterString value: valueString

	parameters add: parameterString -> valueString! !

!PG3StartupMessage methodsFor: 'initialize-release' stamp: 'ul 2/15/2009 18:29'!
initialize

	parameters := OrderedCollection new.
	self initializeParameters ! !

!PG3StartupMessage methodsFor: 'initialize-release' stamp: 'ul 2/16/2009 04:32'!
initializeParameters! !

!PG3StartupMessage methodsFor: 'writing' stamp: 'ul 6/10/2017 23:21'!
writeDataOn: aStream

	aStream nextPutAll: #[0 3 0 0]. "Protocol version 3"
	parameters do: [ :each |
		aStream
			writeString: each key;
			writeString: each value ].
	aStream nextPut: 0 "Packet terminator"! !

PG3Message subclass: #PG3TypedMessage
	instanceVariableNames: 'length'
	classVariableNames: 'DispatcherArray'
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

PG3TypedMessage subclass: #PG3AuthenticationMessage
	instanceVariableNames: ''
	classVariableNames: 'AuthenticationDispatcher'
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

PG3AuthenticationMessage subclass: #PG3AuthenticationCleartextPassword
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3AuthenticationCleartextPassword class methodsFor: 'accessing' stamp: 'ul 2/16/2009 03:11'!
subType

	^3! !

!PG3AuthenticationCleartextPassword methodsFor: 'reading' stamp: 'ul 2/16/2009 03:41'!
readFrom: stream! !

PG3AuthenticationMessage subclass: #PG3AuthenticationCryptPassword
	instanceVariableNames: 'salt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3AuthenticationCryptPassword class methodsFor: 'accessing' stamp: 'ul 2/16/2009 03:11'!
subType

	^4! !

!PG3AuthenticationCryptPassword methodsFor: 'reading' stamp: 'ul 6/11/2017 17:41'!
readFrom: stream

	salt := stream next: 2! !

!PG3AuthenticationCryptPassword methodsFor: 'accessing' stamp: 'ul 6/11/2017 17:42'!
salt

	^salt! !

PG3AuthenticationMessage subclass: #PG3AuthenticationGSS
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3AuthenticationGSS class methodsFor: 'accessing' stamp: 'ul 2/16/2009 03:11'!
subType

	^7! !

PG3AuthenticationMessage subclass: #PG3AuthenticationGSSContinue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3AuthenticationGSSContinue class methodsFor: 'accessing' stamp: 'ul 2/16/2009 03:12'!
subType

	^8! !

PG3AuthenticationMessage subclass: #PG3AuthenticationKerberosV5
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3AuthenticationKerberosV5 class methodsFor: 'accessing' stamp: 'ul 2/16/2009 03:12'!
subType

	^2! !

PG3AuthenticationMessage subclass: #PG3AuthenticationMD5Password
	instanceVariableNames: 'salt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3AuthenticationMD5Password class methodsFor: 'accessing' stamp: 'ul 2/16/2009 03:12'!
subType

	^5! !

!PG3AuthenticationMD5Password methodsFor: 'reading' stamp: 'ul 2/15/2009 22:17'!
readFrom: stream

	salt := stream next: 4! !

!PG3AuthenticationMD5Password methodsFor: 'accessing' stamp: 'ul 2/16/2009 00:44'!
salt

	^salt! !

!PG3AuthenticationMessage class methodsFor: 'class initialization' stamp: 'ul 6/11/2017 22:36'!
initialize

	self initializeAuthenticationDispatcher! !

!PG3AuthenticationMessage class methodsFor: 'class initialization' stamp: 'ul 2/16/2009 04:26'!
initializeAuthenticationDispatcher
	" self initializeAuthenticationDispatcher "

	AuthenticationDispatcher := Array new: 10.
	self allSubclassesDo: [ :subclass |
		AuthenticationDispatcher at: subclass subType + 1 put: subclass ]! !

!PG3AuthenticationMessage class methodsFor: 'instance creation' stamp: 'ul 2/16/2009 04:25'!
readFrom: stream

	| length |
	length := stream nextInt32 - 4.
	^(AuthenticationDispatcher at: stream nextInt32 + 1) new
		length: length;
		readFrom: stream;
		yourself! !

!PG3AuthenticationMessage class methodsFor: 'receiving' stamp: 'ul 2/17/2009 15:16'!
readFrom: stream connection: connection

	| length |
	length := stream nextInt32 - 4.
	^(AuthenticationDispatcher at: stream nextInt32 + 1) new
		connection: connection;
		length: length;
		readFrom: stream;
		yourself! !

!PG3AuthenticationMessage class methodsFor: 'accessing' stamp: 'ul 2/16/2009 03:16'!
subType

	self subclassResponsibility! !

!PG3AuthenticationMessage methodsFor: 'testing' stamp: 'ul 2/15/2009 19:34'!
isResponse

	^true! !

!PG3AuthenticationMessage methodsFor: 'accessing' stamp: 'ul 2/15/2009 00:58'!
typeByte

	^82 "$R asciiValue"! !

PG3AuthenticationMessage subclass: #PG3AuthenticationOkMessage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3AuthenticationOkMessage class methodsFor: 'accessing' stamp: 'ul 2/16/2009 03:12'!
subType

	^0! !

!PG3AuthenticationOkMessage methodsFor: 'reading' stamp: 'ul 2/15/2009 22:17'!
readFrom: stream

	! !

PG3AuthenticationMessage subclass: #PG3AuthenticationSCMCredential
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3AuthenticationSCMCredential class methodsFor: 'accessing' stamp: 'ul 2/16/2009 03:13'!
subType

	^6! !

PG3AuthenticationMessage subclass: #PG3AuthenticationSSPI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3AuthenticationSSPI class methodsFor: 'accessing' stamp: 'ul 2/16/2009 03:13'!
subType

	^9! !

PG3TypedMessage subclass: #PG3BackendKeyData
	instanceVariableNames: 'processId secretKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3BackendKeyData methodsFor: 'testing' stamp: 'ul 2/15/2009 19:34'!
isResponse

	^true! !

!PG3BackendKeyData methodsFor: 'accessing' stamp: 'ul 8/30/2009 18:23'!
processId

	^processId! !

!PG3BackendKeyData methodsFor: 'reading' stamp: 'ul 2/16/2009 05:12'!
readFrom: stream
	
	processId := stream nextInt32.
	secretKey := stream nextInt32! !

!PG3BackendKeyData methodsFor: 'accessing' stamp: 'ul 8/30/2009 18:23'!
secretKey

	^secretKey! !

!PG3BackendKeyData methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:00'!
typeByte

	^75 "$K asciiValue"! !

PG3TypedMessage subclass: #PG3Bind
	instanceVariableNames: 'portalName preparedStatementName parameterFormatCodes parameters resultFormatCodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3Bind methodsFor: 'testing' stamp: 'ul 2/15/2009 19:34'!
isRequest

	^true! !

!PG3Bind methodsFor: 'accessing' stamp: 'ul 6/13/2017 19:52'!
portalName: aString
preparedStatementName: anotherString
parameterFormatCodes: aByteArray
parameters: anArray
resultFormatCodes: anotherByteArray

	portalName := aString.
	preparedStatementName := anotherString.
	parameterFormatCodes := aByteArray.
	parameters := anArray.
	resultFormatCodes := anotherByteArray
	! !

!PG3Bind methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:00'!
typeByte

	^66 "$B asciiValue"! !

!PG3Bind methodsFor: 'writing' stamp: 'ul 6/13/2017 19:49'!
writeDataOn: stream

	| formatCodeCount parameterCount resultFormatCodeCount |
	stream
		writeString: portalName;
		writeString: preparedStatementName;
		writeInt16: (formatCodeCount := parameterFormatCodes size).
	parameterCount := parameters size.
	formatCodeCount = 1
		ifTrue: [ 
			| parameterFormatCode |
			stream
				writeInt16: (parameterFormatCode := parameterFormatCodes at: 1);
				writeInt16: parameterCount.
			1 to: parameterCount do: [ :index |
				stream writeObject: (parameters at: index) formatCode: parameterFormatCode ] ]
		ifFalse: [ 
			parameterCount = formatCodeCount ifFalse: [
				self error: 'Number of parameters and parameterFormatCodes must be the same.' ].
			1 to: formatCodeCount do: [ :index |
				stream writeInt16: (parameterFormatCodes at: index) ].
			stream writeInt16: parameterCount.
			1 to: parameterCount do: [ :index |
				stream writeObject: (parameters at: index) formatCode: (parameterFormatCodes at: index) ] ].
	stream writeInt16: (resultFormatCodeCount := resultFormatCodes size).
	1 to: resultFormatCodeCount do: [ :index |
		stream writeInt16: (resultFormatCodes at: index) ]! !

PG3TypedMessage subclass: #PG3BindComplete
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3BindComplete methodsFor: 'testing' stamp: 'ul 2/15/2009 19:35'!
isResponse

	^true! !

!PG3BindComplete methodsFor: 'reading' stamp: 'ul 6/13/2017 21:14'!
readFrom: aPG3SocketReadStream! !

!PG3BindComplete methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:01'!
typeByte

	^50 "$2 asciiValue"! !

PG3TypedMessage subclass: #PG3Close
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3Close methodsFor: 'testing' stamp: 'ul 2/15/2009 19:35'!
isRequest

	^true! !

!PG3Close methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:04'!
typeByte

	^67 "$C asciiValue"! !

PG3TypedMessage subclass: #PG3CloseComplete
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3CloseComplete methodsFor: 'testing' stamp: 'ul 2/15/2009 19:35'!
isResponse

	^true! !

!PG3CloseComplete methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:05'!
typeByte

	^51 "$3 asciiValue"! !

PG3TypedMessage subclass: #PG3CommandComplete
	instanceVariableNames: 'commandTag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3CommandComplete methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:09'!
commandTag

	^commandTag! !

!PG3CommandComplete methodsFor: 'testing' stamp: 'ul 2/15/2009 19:35'!
isResponse

	^true! !

!PG3CommandComplete methodsFor: 'reading' stamp: 'ul 2/17/2009 02:08'!
readFrom: stream

	commandTag := stream nextString! !

!PG3CommandComplete methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:05'!
typeByte

	^67 "$C asciiValue"! !

PG3TypedMessage subclass: #PG3CopyData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3CopyData methodsFor: 'testing' stamp: 'ul 2/15/2009 19:36'!
isRequest

	^true! !

!PG3CopyData methodsFor: 'testing' stamp: 'ul 2/15/2009 19:36'!
isResponse

	^true! !

!PG3CopyData methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:06'!
typeByte

	^100 "$d asciiValue"! !

PG3TypedMessage subclass: #PG3CopyDone
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3CopyDone methodsFor: 'testing' stamp: 'ul 2/15/2009 19:36'!
isRequest

	^true! !

!PG3CopyDone methodsFor: 'testing' stamp: 'ul 2/15/2009 19:36'!
isResponse

	^true! !

!PG3CopyDone methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:06'!
typeByte

	^99 "$c asciiValue"! !

PG3TypedMessage subclass: #PG3CopyFail
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3CopyFail methodsFor: 'testing' stamp: 'ul 2/15/2009 19:36'!
isRequest

	^true! !

!PG3CopyFail methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:07'!
typeByte

	^102 "$f asciiValue"! !

PG3TypedMessage subclass: #PG3CopyInResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3CopyInResponse methodsFor: 'testing' stamp: 'ul 2/15/2009 19:36'!
isResponse

	^true! !

!PG3CopyInResponse methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:07'!
typeByte

	^71 "$G asciiValue"! !

PG3TypedMessage subclass: #PG3CopyOutResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3CopyOutResponse methodsFor: 'testing' stamp: 'ul 2/15/2009 19:36'!
isResponse

	^true! !

!PG3CopyOutResponse methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:07'!
typeByte

	^72 "$H asciiValue"! !

PG3TypedMessage subclass: #PG3DataRow
	instanceVariableNames: 'values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3DataRow methodsFor: 'testing' stamp: 'ul 2/16/2009 02:29'!
isResponse

	^true! !

!PG3DataRow methodsFor: 'reading' stamp: 'klub 8/10/2015 12:21'!
readFrom: stream

	| converters remainingBytes |
	length < 2 ifTrue: [
		PG3SocketReadStreamError
			signal: 'At least 2 bytes expected, but only {1} is available.'
			format: { length } ].
	values := connection currentRowClass new: stream nextInt16.
	converters := connection currentConverterSelectors.
	remainingBytes := length - 2.
	1 to: values size do: [ :index |
		| valueLength |
		valueLength := stream nextInt32.
		remainingBytes := remainingBytes - 4.
		valueLength > remainingBytes ifTrue: [ 
			PG3SocketReadStreamError
				signal: 'Expected {1} bytes, but only got {2}.'
				format: { valueLength. remainingBytes } ].
		values at: index put: (
			valueLength = -1
				ifTrue: [ nil ]
				ifFalse: [ stream perform: (converters at: index) with: valueLength ]).
		valueLength > 0 ifTrue: [ remainingBytes := remainingBytes - valueLength ] ].
	remainingBytes = 0 ifTrue: [ ^self ].
	PG3SocketReadStreamError
		signal: '{1} unread bytes have remained.'
		format: { remainingBytes }! !

!PG3DataRow methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:08'!
typeByte

	^68 "$D asciiValue"! !

!PG3DataRow methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:31'!
values

	^values! !

PG3TypedMessage subclass: #PG3Describe
	instanceVariableNames: 'type name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3Describe methodsFor: 'testing' stamp: 'ul 2/16/2009 02:34'!
isRequest

	^true! !

!PG3Describe methodsFor: 'accessing' stamp: 'ul 6/13/2017 23:22'!
portalNamed: aString

	type :=  80. "$P asciiValue"
	name := aString! !

!PG3Describe methodsFor: 'accessing' stamp: 'ul 6/13/2017 23:22'!
statementNamed: aString

	type :=  83. "$S asciiValue"
	name := aString! !

!PG3Describe methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:08'!
typeByte

	^68 "$D asciiValue"! !

!PG3Describe methodsFor: 'as yet unclassified' stamp: 'ul 6/13/2017 23:22'!
writeDataOn: stream

	stream 
		nextPut: type;
		writeString: name! !

PG3TypedMessage subclass: #PG3EmptyQueryResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3EmptyQueryResponse methodsFor: 'testing' stamp: 'ul 2/16/2009 02:34'!
isResponse

	^true! !

!PG3EmptyQueryResponse methodsFor: 'reading' stamp: 'ul 2/17/2009 02:22'!
readFrom: stream! !

!PG3EmptyQueryResponse methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:09'!
typeByte

	^73 "$I asciiValue"! !

PG3TypedMessage subclass: #PG3ExceptionResponse
	instanceVariableNames: 'severity code message detail hint position internalPosition internalQuery where file routine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

PG3ExceptionResponse subclass: #PG3ErrorResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3ErrorResponse methodsFor: 'accessing' stamp: 'ul 2/26/2009 14:44'!
typeByte

	^69 "$E asciiValue"! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
code
	^ code! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
code: anObject
	^ code := anObject! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
detail
	^ detail! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
detail: anObject
	^ detail := anObject! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
file
	^ file! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
file: anObject
	^ file := anObject! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
hint
	^ hint! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
hint: anObject
	^ hint := anObject! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
internalPosition
	^ internalPosition! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
internalPosition: anObject
	^ internalPosition := anObject! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
internalQuery
	^ internalQuery! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
internalQuery: anObject
	^ internalQuery := anObject! !

!PG3ExceptionResponse methodsFor: 'testing' stamp: 'ul 2/26/2009 14:44'!
isResponse

	^true! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
message
	^ message! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
message: anObject
	^ message := anObject! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
position
	^ position! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
position: anObject
	^ position := anObject! !

!PG3ExceptionResponse methodsFor: 'reading' stamp: 'ul 2/16/2009 04:56'!
readFrom: aStream

	[
		aStream next
			caseOf: {
				[ 0 ] -> [ ^self ].
				[ $S asciiValue ] -> [ severity := aStream nextString ].
				[ $C asciiValue ] -> [ code := aStream nextString ].
				[ $M asciiValue ] -> [ message := aStream nextString ].
				[ $D asciiValue ] -> [ detail := aStream nextString ].
				[ $H asciiValue ] -> [ hint := aStream nextString ].
				[ $P asciiValue ] -> [ position := aStream nextString ].
				[ $p asciiValue ] -> [ internalPosition := aStream nextString ].
				[ $q asciiValue ] -> [ internalQuery := aStream nextString ].
				[ $W asciiValue ] -> [ where := aStream nextString ].
				[ $F asciiValue ] -> [ file := aStream nextString ].
				[ $R asciiValue ] -> [ routine := aStream nextString ] }
			otherwise: [ aStream nextString ] ] repeat! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
routine
	^ routine! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
routine: anObject
	^ routine := anObject! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
severity
	^ severity! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
severity: anObject
	^ severity := anObject! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
where
	^ where! !

!PG3ExceptionResponse methodsFor: 'accessing' stamp: 'ul 2/16/2009 04:56'!
where: anObject
	^ where := anObject! !

PG3ExceptionResponse subclass: #PG3NoticeResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3NoticeResponse methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:45'!
typeByte

	^78 "$N asciiValue"! !

PG3TypedMessage subclass: #PG3Execute
	instanceVariableNames: 'portalName maxRows'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3Execute methodsFor: 'accessing' stamp: 'ul 6/13/2017 23:01'!
executePortal: aString maxRows: anInteger

	portalName := aString.
	maxRows := anInteger! !

!PG3Execute methodsFor: 'testing' stamp: 'ul 2/16/2009 02:35'!
isRequest

	^true! !

!PG3Execute methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:25'!
typeByte

	^69 "$I asciiValue"! !

!PG3Execute methodsFor: 'writing' stamp: 'ul 6/13/2017 23:02'!
writeDataOn: stream

	stream
		writeString: portalName;
		writeInt32: maxRows! !

PG3TypedMessage subclass: #PG3Flush
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3Flush methodsFor: 'testing' stamp: 'ul 2/16/2009 02:35'!
isRequest

	^true! !

!PG3Flush methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:44'!
typeByte

	^72 "$H asciiValue"! !

PG3TypedMessage subclass: #PG3FunctionCall
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3FunctionCall methodsFor: 'testing' stamp: 'ul 2/16/2009 02:37'!
isRequest

	^true! !

!PG3FunctionCall methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:44'!
typeByte

	^70 "$F asciiValue"! !

PG3TypedMessage subclass: #PG3FunctionCallResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3FunctionCallResponse methodsFor: 'testing' stamp: 'ul 2/16/2009 02:37'!
isResponse

	^true! !

!PG3FunctionCallResponse methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:45'!
typeByte

	^86 "$V asciiValue"! !

PG3TypedMessage subclass: #PG3NoData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3NoData methodsFor: 'testing' stamp: 'ul 2/16/2009 02:39'!
isResponse

	^true! !

!PG3NoData methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:45'!
typeByte

	^110 "$n asciiValue"! !

PG3TypedMessage subclass: #PG3NotificationResponse
	instanceVariableNames: 'processId conditionName payload'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3NotificationResponse methodsFor: 'accessing' stamp: 'kb 8/25/2009 09:48'!
conditionName

	^conditionName ! !

!PG3NotificationResponse methodsFor: 'testing' stamp: 'ul 2/16/2009 02:39'!
isResponse

	^true! !

!PG3NotificationResponse methodsFor: 'accessing' stamp: 'klub 6/2/2014 17:04'!
payload

	^payload! !

!PG3NotificationResponse methodsFor: 'accessing' stamp: 'kb 8/25/2009 09:48'!
processId

	^processId! !

!PG3NotificationResponse methodsFor: 'reading' stamp: 'klub 6/2/2014 17:05'!
readFrom: stream

	processId := stream nextInt32.
	conditionName := stream nextString.
	payload := stream nextString! !

!PG3NotificationResponse methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:46'!
typeByte

	^65 "$A asciiValue"! !

PG3TypedMessage subclass: #PG3ParameterDescription
	instanceVariableNames: 'types'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3ParameterDescription methodsFor: 'testing' stamp: 'ul 2/16/2009 02:39'!
isResponse

	^true! !

!PG3ParameterDescription methodsFor: 'reading' stamp: 'ul 6/14/2017 10:25'!
readFrom: stream

	| length |
	length := stream nextInt16.
	types := Array new: length.
	1 to: length do: [ :index |
		types at: index put: stream nextInt32 ]! !

!PG3ParameterDescription methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:46'!
typeByte

	^116 "$t asciiValue"! !

PG3TypedMessage subclass: #PG3ParameterStatus
	instanceVariableNames: 'name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3ParameterStatus methodsFor: 'testing' stamp: 'ul 2/16/2009 02:40'!
isResponse

	^true! !

!PG3ParameterStatus methodsFor: 'testing' stamp: 'ul 2/16/2009 05:00'!
name

	^name! !

!PG3ParameterStatus methodsFor: 'reading' stamp: 'ul 2/16/2009 04:59'!
readFrom: stream

	name := stream nextString.
	value := stream nextString! !

!PG3ParameterStatus methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:46'!
typeByte

	^83 "$S asciiValue"! !

!PG3ParameterStatus methodsFor: 'evaluating' stamp: 'ul 2/16/2009 05:00'!
value

	^value! !

PG3TypedMessage subclass: #PG3Parse
	instanceVariableNames: 'preparedStatementName queryString parameterTypes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3Parse methodsFor: 'testing' stamp: 'ul 2/16/2009 02:40'!
isRequest

	^true! !

!PG3Parse methodsFor: 'accessing' stamp: 'ul 6/11/2017 18:14'!
preparedStatementName: aString queryString: anotherString parameterTypes: anArray

	preparedStatementName := aString.
	queryString := anotherString.
	parameterTypes := anArray! !

!PG3Parse methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:46'!
typeByte

	^80 "$P asciiValue"! !

!PG3Parse methodsFor: 'writing' stamp: 'ul 6/12/2017 03:09'!
writeDataOn: stream

	stream
		writeString: preparedStatementName;
		writeString: queryString;
		writeInt16: parameterTypes size.
	1 to: parameterTypes size do: [ :each |
		stream writeInt32: (parameterTypes at: each) ]
		! !

PG3TypedMessage subclass: #PG3ParseComplete
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3ParseComplete methodsFor: 'testing' stamp: 'ul 2/16/2009 02:40'!
isResponse

	^true! !

!PG3ParseComplete methodsFor: 'reading' stamp: 'ul 6/12/2017 02:12'!
readFrom: stream! !

!PG3ParseComplete methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:47'!
typeByte

	^49 "$1 asciiValue"! !

PG3TypedMessage subclass: #PG3PasswordMessage
	instanceVariableNames: 'password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3PasswordMessage methodsFor: 'testing' stamp: 'ul 2/16/2009 02:40'!
isRequest

	^true! !

!PG3PasswordMessage methodsFor: 'as yet unclassified' stamp: 'ul 2/16/2009 00:46'!
password: aString

	password := aString	

	! !

!PG3PasswordMessage methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:47'!
typeByte

	^112 "$p asciiValue"! !

!PG3PasswordMessage methodsFor: 'writing' stamp: 'ul 6/10/2017 23:16'!
writeDataOn: aStream

	aStream writeString: password! !

PG3TypedMessage subclass: #PG3PortalSuspended
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3PortalSuspended methodsFor: 'testing' stamp: 'ul 2/16/2009 02:40'!
isResponse

	^true! !

!PG3PortalSuspended methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:47'!
typeByte

	^115 "$s asciiValue"! !

PG3TypedMessage subclass: #PG3Query
	instanceVariableNames: 'queryString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3Query methodsFor: 'testing' stamp: 'ul 2/16/2009 02:40'!
isRequest

	^true! !

!PG3Query methodsFor: 'as yet unclassified' stamp: 'ul 2/17/2009 02:07'!
queryString: aByteString

	queryString := aByteString! !

!PG3Query methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:48'!
typeByte

	^81 "$Q asciiValue"! !

!PG3Query methodsFor: 'writing' stamp: 'ul 6/10/2017 23:20'!
writeDataOn: aStream

	aStream writeString: queryString! !

PG3TypedMessage subclass: #PG3ReadyForQuery
	instanceVariableNames: 'transactionState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3ReadyForQuery methodsFor: 'testing' stamp: 'ul 2/16/2009 02:41'!
isResponse

	^true! !

!PG3ReadyForQuery methodsFor: 'reading' stamp: 'ul 2/16/2009 02:21'!
readFrom: stream

	transactionState := stream next! !

!PG3ReadyForQuery methodsFor: 'as yet unclassified' stamp: 'ul 2/16/2009 02:21'!
transactionState

	^transactionState! !

!PG3ReadyForQuery methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:48'!
typeByte

	^90 "$Z asciiValue"! !

PG3TypedMessage subclass: #PG3RowDescription
	instanceVariableNames: 'columnDescriptions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3RowDescription methodsFor: 'accessing' stamp: 'ul 9/5/2009 21:30'!
columnNameDictionary

	| dictionary |
	dictionary := IdentityDictionary new: columnDescriptions size * 2.
	columnDescriptions doWithIndex: [ :columnDescription :index |
		dictionary at: columnDescription name asSymbol put: index ].
	^dictionary! !

!PG3RowDescription methodsFor: 'accessing' stamp: 'ul 2/17/2009 02:21'!
columnsDescriptions

	^columnDescriptions! !

!PG3RowDescription methodsFor: 'accessing' stamp: 'ul 2/26/2009 11:57'!
converterSelectors

	^columnDescriptions collect: [ :each | each converterSelector ]! !

!PG3RowDescription methodsFor: 'testing' stamp: 'ul 2/16/2009 02:41'!
isResponse

	^true! !

!PG3RowDescription methodsFor: 'reading' stamp: 'ul 2/26/2009 00:31'!
readFrom: stream

	columnDescriptions := Array new: stream nextInt16.
	1 to: columnDescriptions size do: [ :index |
		columnDescriptions at: index put: (
			PG3ColumnDescription new
				name: stream nextString;
				tableId: stream nextInt32;
				columnId: stream nextInt16;
				typeId: stream nextInt32;
				typeSize: stream nextInt16;
				typeModifier: stream nextInt32;
				formatCode: stream nextInt16;
				yourself) ]! !

!PG3RowDescription methodsFor: 'accessing' stamp: 'ul 2/15/2009 01:48'!
typeByte

	^84 "$T asciiValue"! !

PG3TypedMessage subclass: #PG3Sync
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3Sync methodsFor: 'testing' stamp: 'ul 2/16/2009 02:41'!
isRequest

	^true! !

!PG3Sync methodsFor: 'accessing' stamp: 'ul 2/15/2009 00:00'!
typeByte

	^83 "$S asciiValue"! !

PG3TypedMessage subclass: #PG3Terminate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core-Messages'!

!PG3Terminate methodsFor: 'testing' stamp: 'ul 2/16/2009 02:41'!
isRequest

	^true! !

!PG3Terminate methodsFor: 'accessing' stamp: 'ul 2/14/2009 23:59'!
typeByte

	^88 "$X asciiValue"! !

!PG3TypedMessage class methodsFor: 'receiving' stamp: 'ul 2/17/2009 15:14'!
dispatchFrom: stream connection: connection

	^(DispatcherArray at: stream next) readFrom: stream connection: connection! !

!PG3TypedMessage class methodsFor: 'class initialization' stamp: 'ul 6/11/2017 22:35'!
initialize

	self initializeDispatcherArray ! !

!PG3TypedMessage class methodsFor: 'class initialization' stamp: 'ul 2/16/2009 02:59'!
initializeDispatcherArray
	" self initializeDispatcherArray "

	DispatcherArray := Array new: 256.
	self allSubclassesDo: [ :subclass |
		subclass basicNew in: [ :instance |
			((subclass methodDictionary includesKey: #typeByte) and: [ 
				instance isResponse ]) ifTrue: [
					DispatcherArray at: instance typeByte put: subclass ] ] ]! !

!PG3TypedMessage class methodsFor: 'instance creation' stamp: 'ul 2/16/2009 03:04'!
readFrom: stream

	^self new
		length: stream nextInt32 - 4;
		readFrom: stream;
		yourself! !

!PG3TypedMessage class methodsFor: 'receiving' stamp: 'ul 2/17/2009 15:16'!
readFrom: stream connection: connection

	^self new
		connection: connection;
		length: stream nextInt32 - 4;
		readFrom: stream;
		yourself! !

!PG3TypedMessage methodsFor: 'testing' stamp: 'ul 2/15/2009 19:33'!
isRequest

	^false! !

!PG3TypedMessage methodsFor: 'testing' stamp: 'ul 2/15/2009 19:34'!
isResponse

	^false! !

!PG3TypedMessage methodsFor: 'accessing' stamp: 'ul 2/15/2009 22:08'!
length: anInteger

	length := anInteger! !

!PG3TypedMessage methodsFor: 'reading' stamp: 'ul 2/15/2009 22:10'!
readFrom: stream

	self subclassResponsibility! !

!PG3TypedMessage methodsFor: 'accessing' stamp: 'klub 7/7/2011 16:53'!
typeByte

	self subclassResponsibility! !

!PG3TypedMessage methodsFor: 'writing' stamp: 'ul 6/10/2017 23:12'!
writeOn: aStream

	aStream nextPut: self typeByte.
	super writeOn: aStream! !

Object subclass: #PG3ResultSet
	instanceVariableNames: 'rowDescription rows'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3ResultSet commentStamp: 'ul 5/14/2010 21:11' prior: 0!
A PG3ResultSet represents the result of a single query. It contains the description of the rows and the rows themselves.

Instance Variables
	rowDescription:		<PG3RowDescription>
	rows:		<SequenceableCollection>

rowDescription
	- this object describes the structure of the rows.

rows
	- the rows returned by the query. It can be empty.
!

!PG3ResultSet methodsFor: 'accessing' stamp: 'ul 9/5/2009 21:41'!
rowDescription

	^rowDescription! !

!PG3ResultSet methodsFor: 'accessing' stamp: 'ul 9/5/2009 21:40'!
rowDescription: aPG3RowDescription

	rowDescription := aPG3RowDescription! !

!PG3ResultSet methodsFor: 'accessing' stamp: 'ul 9/5/2009 21:39'!
rows

	^rows! !

!PG3ResultSet methodsFor: 'accessing' stamp: 'ul 5/14/2010 21:10'!
rows: aSequenceableCollection

	rows := aSequenceableCollection! !

!PG3ResultSet methodsFor: 'accessing' stamp: 'ul 5/14/2010 21:09'!
rowsWithColumnNameDictionary
	"Pass the columnNameDictionary to all rows then return the rows."
	
	| columnNameDictionary |
	columnNameDictionary := rowDescription columnNameDictionary.
	rows do: [ :each | 
		each columnNameDictionary: columnNameDictionary ].
	^rows
! !

Object variableSubclass: #PG3Row
	instanceVariableNames: 'columnNameDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3Row commentStamp: 'ul 4/14/2010 03:57' prior: 0!
A PG3Row is an Array-like object which represents a row returned by a query. It can be accessed by index (via #at: and #at:put:) or by the name of the column sent to it as a unary message, if columnNameDictionary is not nil, the name of the column is a valid unary selector and the selector is not defined by Object or ProtoObject (the only exception is #name, which can be used as a column name). If the column name is not a valid unary selector or the selector is implemented by a superclass (Object or ProtoObject), then #atName: #atName:ifAbsent: and #atName:put: can be used to access the slots by name.

Instance Variables
	columnNameDictionary:		<IdentityDictionary>

columnNameDictionary
	- the IdentityDictionary which maps column names to indices, it's size should be the same as the PG3Row's size, it's keys should be symbols representing the column names and it's values should be integers between 1 the size of the row. All integers between 1 and row size should be present as a value.!

!PG3Row class methodsFor: 'as yet unclassified' stamp: 'ul 9/5/2009 21:23'!
for: anIdentityDictionary

	^(self new: anIdentityDictionary size)
		columnNameDictionary: anIdentityDictionary;
		yourself! !

!PG3Row methodsFor: 'accessing' stamp: 'ul 9/5/2009 21:23'!
atName: aSymbol

	^self at: (columnNameDictionary at: aSymbol)! !

!PG3Row methodsFor: 'accessing' stamp: 'ul 9/5/2009 21:23'!
atName: aSymbol ifAbsent: aBlock

	^self at: (columnNameDictionary 
		at: aSymbol
		ifAbsent: [ ^aBlock value ])! !

!PG3Row methodsFor: 'accessing' stamp: 'kb 3/9/2010 23:38'!
atName: aSymbol put: anObject

	^self at: (columnNameDictionary at: aSymbol) put: anObject! !

!PG3Row methodsFor: 'accessing' stamp: 'ul 9/13/2009 19:58'!
columnNameDictionary

	^columnNameDictionary! !

!PG3Row methodsFor: 'accessing' stamp: 'ul 9/5/2009 21:23'!
columnNameDictionary: anIdentityDictionary

	columnNameDictionary := anIdentityDictionary! !

!PG3Row methodsFor: 'accessing' stamp: 'ul 4/14/2010 03:40'!
doesNotUnderstand: aMessage

	^(columnNameDictionary 
		at: aMessage selector
		ifAbsent: nil)
			ifNil: [ super doesNotUnderstand: aMessage ]
			ifNotNil: [ :index | self at: index ]! !

!PG3Row methodsFor: 'explorer' stamp: 'klub 6/7/2014 17:34'!
explorerContents

	^Smalltalk at: #ObjectExplorerWrapper ifPresent: [ :objectExplorerWrapper |
		| instVarNames |
		instVarNames := self class allInstVarNames.
		Array new: self size + instVarNames size streamContents: [ :stream |
			instVarNames withIndexDo: [ :name :index | 
				| value |
				value := self instVarAt: index.
				stream nextPut: (
					objectExplorerWrapper
						with: value
						name: name
						model: value) ].
			columnNameDictionary 
				ifNil: [
					1 to: self size do: [ :index |
						stream nextPut: (objectExplorerWrapper
							with: (self at: index)
							name: index printString
							model: self) ] ]
				ifNotNil: [
					stream nextPutAll: (
						(columnNameDictionary associations
							sort: [ :a :b | a value < b value ])
							replace: [ :assoc |
								objectExplorerWrapper
									with: (self at: assoc value)
									name: assoc key, (' (', assoc value asString, ')')
									model: self ]) ] ] ]! !

!PG3Row methodsFor: 'accessing' stamp: 'ul 9/5/2009 21:24'!
indexOf: aSymbol

	^columnNameDictionary at: aSymbol ifAbsent: 0! !

!PG3Row methodsFor: 'enumerating' stamp: 'ul 9/6/2009 15:26'!
keysAndValuesDo: aBlock

	columnNameDictionary keysAndValuesDo: [ :key :value |
		aBlock value: key value: (self at: value) ]! !

!PG3Row methodsFor: 'accessing' stamp: 'ul 4/14/2010 03:38'!
name

	^self atName: #name ifAbsent: [ super name ]! !

!PG3Row methodsFor: 'printing' stamp: 'ul 9/5/2009 21:37'!
printOn: aStream

	super printOn: aStream.
	aStream nextPut: $(.
	columnNameDictionary 
		ifNil: [
			1 to: self size do: [ :index |
				index = 1 ifFalse: [ aStream nextPutAll: ', ' ].
				aStream print: (self at: index) ] ]
		ifNotNil: [
			(columnNameDictionary associations
				sort: [ :a :b | a value < b value ])
				do: [ :assoc |
					aStream 
						nextPutAll: assoc key;
						nextPutAll: ': ';
						print: (self at: assoc value) ]
				separatedBy: [ aStream nextPutAll: ', ' ] ].
	aStream nextPut: $)! !

Object subclass: #PG3ServerState
	instanceVariableNames: 'transitions name stopState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3ServerState class methodsFor: 'protocols-startup' stamp: 'ul 6/11/2017 17:43'!
authenticatingStateDescription

	(self state: #Authenticating)
		on: PG3AuthenticationOkMessage goto: #InitializingBackend;
		on: PG3AuthenticationMD5Password 
			connectionDo: #respondToAuthenticationMD5PasswordRequest:
			goto: #AuthenticatingWithMD5;
		on: PG3AuthenticationCryptPassword
			connectionDo: #respondToAuthenticationCryptPasswordRequest:
			goto: #AuthenticatingWithCrypt;
		on: PG3AuthenticationCleartextPassword
			connectionDo: #respondToAuthenticationClearTextRequest:
			goto: #AuthenticatingWithClearText! !

!PG3ServerState class methodsFor: 'protocols-startup' stamp: 'ul 6/10/2017 23:29'!
authenticatingWithClearTextStateDescription

	(self state: #AuthenticatingWithClearText)
		on: PG3AuthenticationOkMessage goto: #InitializingBackend! !

!PG3ServerState class methodsFor: 'protocols-startup' stamp: 'ul 6/11/2017 17:43'!
authenticatingWithCryptStateDescription

	(self state: #AuthenticatingWithCrypt)
		on: PG3AuthenticationOkMessage goto: #InitializingBackend! !

!PG3ServerState class methodsFor: 'protocols-startup' stamp: 'ul 2/16/2009 00:53'!
authenticatingWithMD5StateDescription

	(self state: #AuthenticatingWithMD5)
		on: PG3AuthenticationOkMessage goto: #InitializingBackend! !

!PG3ServerState class methodsFor: 'protocols-extended query' stamp: 'ul 6/13/2017 23:46'!
bindCompletedState

	(self state: #BindCompleted)
		on: PG3RowDescription "After Describe portal"
			connectionDo: #rowDescription:
			goto: #DescribeCompleted;
		on: PG3DataRow "When no Describe was issued"
			connectionDo: #dataRow:
			goto: #ReceivingData;
		on: PG3NoData "After Describe portal, when the query won't return anything."
			goto: #DescribeCompleted;
		on: PG3ErrorResponse "No such portal"
			connectionDo: #handleError:
			goto: #DescribeCompleted;
		on: PG3ReadyForQuery "After a Sync message."
			connectionDo: #readyForQuery:
			goto: #WaitingForQuery! !

!PG3ServerState class methodsFor: 'protocols-extended query' stamp: 'ul 6/13/2017 21:08'!
bindingState

	(self state: #Binding)
		on: PG3BindComplete
		connectionDo: #bindComplete:
		goto: #BindCompleted;
		on: PG3ErrorResponse 
		connectionDo: #handleError:
		goto: #WaitingForQuery! !

!PG3ServerState class methodsFor: 'state machine creation' stamp: 'ul 2/17/2009 02:49'!
createStateGraph

	| states |
	states := IdentityDictionary new.
	[ self stateGraphDescription ]
		on: Notification
		do: [ :notification | notification resume: states ].
	^states! !

!PG3ServerState class methodsFor: 'protocols-extended query' stamp: 'ul 6/13/2017 23:49'!
describeCompletedState

	(self state: #DescribeCompleted)
		on: PG3DataRow
			connectionDo: #dataRow:
			goto: #ReceivingData;
		on: PG3EmptyQueryResponse
			goto: #GotEmptyQueryResponse;			
		on: PG3ReadyForQuery
			connectionDo: #readyForQuery:
			goto: #WaitingForQuery! !

!PG3ServerState class methodsFor: 'protocols-extended query' stamp: 'ul 6/13/2017 23:10'!
executeCompletedState

	(self state: #ExecuteCompleted)
		on: PG3PortalSuspended
		goto: #WaitingForQuery;
		on: PG3ReadyForQuery
		connectionDo: #readyForQuery:
		goto: #WaitingForQuery;
		on: PG3ErrorResponse 
		connectionDo: #handleError:
		goto: #GotErrorResponseDuringSimpleQuery! !

!PG3ServerState class methodsFor: 'protocols-extended query' stamp: 'ul 6/13/2017 23:49'!
executingState

	(self state: #Executing)
		on: PG3CommandComplete
			connectionDo: #commandComplete:
			goto: #GotCommandComplete;
		on: PG3EmptyQueryResponse
			goto: #GotEmptyQueryResponse;
		on: PG3ErrorResponse 
			connectionDo: #handleError:
			goto: #GotErrorResponseDuringSimpleQuery! !

!PG3ServerState class methodsFor: 'protocols-simple query' stamp: 'ul 3/6/2009 03:49'!
gotCommandComplete

	(self state: #GotCommandComplete)
		on: PG3CommandComplete
		connectionDo: #commandComplete:
		goto: #GotCommandComplete;
		on: PG3ReadyForQuery
		connectionDo: #readyForQuery:
		goto: #WaitingForQuery;
		on: PG3RowDescription
		connectionDo: #rowDescription:
		goto: #GotRowDescription;
		on: PG3ErrorResponse 
		connectionDo: #handleError:
		goto: #GotErrorResponseDuringSimpleQuery! !

!PG3ServerState class methodsFor: 'protocols-simple query' stamp: 'ul 3/6/2009 03:49'!
gotDataRow

	(self state: #GotDataRow)
		on: PG3DataRow
		connectionDo: #dataRow:
		goto: #GotDataRow;
		on: PG3CommandComplete
		connectionDo: #commandComplete:
		goto: #GotCommandComplete;
		on: PG3ErrorResponse 
		connectionDo: #handleError:
		goto: #GotErrorResponseDuringSimpleQuery! !

!PG3ServerState class methodsFor: 'protocols-simple query' stamp: 'ul 3/6/2009 03:49'!
gotEmptyQueryResponse

	(self state: #GotEmptyQueryResponse)
		on: PG3ReadyForQuery
		connectionDo: #readyForQuery:
		goto: #WaitingForQuery;
		on: PG3ErrorResponse 
		connectionDo: #handleError:
		goto: #GotErrorResponseDuringSimpleQuery! !

!PG3ServerState class methodsFor: 'protocols-simple query' stamp: 'ul 3/6/2009 03:48'!
gotErrorResponseDuringSimpleQuery

	(self state: #GotErrorResponseDuringSimpleQuery)
		on: PG3ReadyForQuery
		connectionDo: #readyForQuery:
		goto: #WaitingForQuery! !

!PG3ServerState class methodsFor: 'protocols-simple query' stamp: 'ul 9/7/2009 00:33'!
gotRowDescription

	(self state: #GotRowDescription)
		on: PG3DataRow
		connectionDo: #dataRow:
		goto: #GotDataRow;
		on: PG3CommandComplete
		connectionDo: #commandComplete:
		goto: #GotCommandComplete;
		on: PG3ErrorResponse 
		connectionDo: #handleError:
		goto: #GotErrorResponseDuringSimpleQuery! !

!PG3ServerState class methodsFor: 'protocols-startup' stamp: 'ul 6/10/2017 23:29'!
initializingBackendStateDescription

	(self state: #InitializingBackend)
		on: PG3BackendKeyData
			connectionDo: #registerBackendKeyData:
			goto: #InitializingBackend;
		on: PG3ReadyForQuery
			connectionDo: #readyForQuery:
			goto: #WaitingForQuery! !

!PG3ServerState class methodsFor: 'protocols-extended query' stamp: 'ul 6/12/2017 20:22'!
parseCompleted

	(self state: #ParseCompleted)
		on: PG3ReadyForQuery
		connectionDo: #readyForQuery:
		goto: #WaitingForQuery! !

!PG3ServerState class methodsFor: 'protocols-extended query' stamp: 'ul 6/12/2017 20:22'!
parsing

	(self state: #Parsing)
		on: PG3ParseComplete
		connectionDo: #parseComplete:
		goto: #ParseCompleted;
		on: PG3ErrorResponse 
		connectionDo: #handleError:
		goto: #WaitingForQuery! !

!PG3ServerState class methodsFor: 'protocols-simple query' stamp: 'ul 3/6/2009 03:49'!
querying

	(self state: #Querying)
		on: PG3CommandComplete
		connectionDo: #commandComplete:
		goto: #GotCommandComplete;
		on: PG3RowDescription
		connectionDo: #rowDescription:
		goto: #GotRowDescription;
		on: PG3EmptyQueryResponse
		goto: #GotEmptyQueryResponse;
		on: PG3ErrorResponse 
		connectionDo: #handleError:
		goto: #GotErrorResponseDuringSimpleQuery! !

!PG3ServerState class methodsFor: 'protocols-extended query' stamp: 'ul 6/13/2017 23:47'!
receivingDataState

	(self state: #ReceivingData)
		on: PG3DataRow "When no Describe was issued"
			connectionDo: #dataRow:
			goto: #ReceivingData;
		on: PG3CommandComplete
			connectionDo: #commandComplete:
			goto: #GotCommandComplete;
		on: PG3ErrorResponse "No such portal"
			connectionDo: #handleError:
			goto: #GotErrorResponseDuringSimpleQuery;
		on: PG3ReadyForQuery "After a Sync message."
			connectionDo: #readyForQuery:
			goto: #WaitingForQuery! !

!PG3ServerState class methodsFor: 'state machine creation' stamp: 'ul 6/10/2017 23:32'!
state: aSymbol

	^self states at: aSymbol ifAbsent: [
		| newState |
		newState := self states at: aSymbol put: (
			self new
				name: aSymbol;
				yourself).
		newState
			on: PG3NoticeResponse
			connectionDo: #receiveNotice:
			goto: aSymbol;
			on: PG3ParameterStatus 
			connectionDo: #addParameterStatus:
			goto: aSymbol;
			on: PG3NotificationResponse  
			connectionDo: #receiveNotification:
			goto: aSymbol;
			yourself ].! !

!PG3ServerState class methodsFor: 'state machine creation' stamp: 'ul 6/13/2017 23:48'!
stateGraphDescription

	self 
		"startup"
		authenticatingStateDescription;
		authenticatingWithMD5StateDescription;
		authenticatingWithClearTextStateDescription;
		authenticatingWithCryptStateDescription;
		initializingBackendStateDescription;
		"general"
		waitingForQuery;
		"simple query"
		querying;
		gotCommandComplete;
		gotRowDescription;
		gotDataRow;
		gotEmptyQueryResponse;
		gotErrorResponseDuringSimpleQuery;
		"extended query"
		parsing;
		parseCompleted;
		bindingState;
		bindCompletedState;
		describeCompletedState;
		receivingDataState;
		executingState;
		executeCompletedState! !

!PG3ServerState class methodsFor: 'state machine creation' stamp: 'ul 2/16/2009 01:00'!
states

	^Notification signal! !

!PG3ServerState class methodsFor: 'protocols-startup' stamp: 'ul 6/10/2017 23:42'!
waitingForQuery

	(self state: #WaitingForQuery)
		beStopState! !

!PG3ServerState methodsFor: 'accessing' stamp: 'ul 6/10/2017 23:43'!
beStopState
	"Make this state be a stop state. Messages are processed until such state is reached."

	stopState := true! !

!PG3ServerState methodsFor: 'initialize-release' stamp: 'ul 2/26/2009 15:42'!
initialize

	transitions := IdentityDictionary new.
	stopState := false! !

!PG3ServerState methodsFor: 'testing' stamp: 'ul 2/26/2009 15:43'!
isStopState

	^stopState! !

!PG3ServerState methodsFor: 'testing' stamp: 'ul 2/16/2009 05:19'!
name

	^name! !

!PG3ServerState methodsFor: 'accessing' stamp: 'ul 2/16/2009 01:08'!
name: aSymbol

	name := aSymbol! !

!PG3ServerState methodsFor: 'transitions' stamp: 'ul 2/16/2009 11:47'!
on: aPG3MessageClass connectionDo: selector goto: aSymbol

	transitions at: aPG3MessageClass put: { (self state: aSymbol). selector }! !

!PG3ServerState methodsFor: 'transitions' stamp: 'ul 2/16/2009 01:07'!
on: aPG3MessageClass goto: aSymbol

	transitions at: aPG3MessageClass put: { (self state: aSymbol). nil }! !

!PG3ServerState methodsFor: 'printing' stamp: 'ul 3/6/2009 03:55'!
printOn: aStream

	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: name;
		nextPut: $)! !

!PG3ServerState methodsFor: 'processing' stamp: 'ul 6/10/2017 23:40'!
receive: aPG3TypedMessage from: connection

	| action |
	action := transitions at: aPG3TypedMessage class ifAbsent: nil.
	action ifNil: [ self error: 'Missing transition' ].
	(action at: 2) ifNotNil: [ :actionSelector |
		connection 
			perform: actionSelector 
			with: aPG3TypedMessage ].
	^action at: 1! !

!PG3ServerState methodsFor: 'accessing' stamp: 'ul 2/16/2009 01:06'!
state: aSymbol

	^self class state: aSymbol! !

Object subclass: #PG3SocketReadStream
	instanceVariableNames: 'buffer position readLimit socket textConverter offset bytesToKeep jsonParser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3SocketReadStream commentStamp: 'klub 8/10/2015 13:07' prior: 0!
I'm implemet ReadStream-like objects which can hold data temporarily from the network in a ByteArray

Instance Variables
	buffer:		<ByteArray>
	bytesToKeep:		<Integer>
	offset:		<Integer>
	position:		<Integer>
	readLimit:		<Integer>
	socket:		<Socket>
	textConverter:		<PG3TextConverter>

buffer
	- a ByteArray holding the bytes read from the network

bytesToKeep
	- the number of bytes to keep for read ahead. When the buffer is refilled from the network, this number of bytes will be kept from the previous read. It must be smaller than the size of the buffer, otherwise the bytes can't be kept. 

offset
	- the total number of bytes read from the network since the last #resetOffset send, not including the bytes currently present in the buffer.  

position
	- the last read position of the buffer. #next will return the byte from the buffer at position + 1.

readLimit
	- the last position in buffer which contains a byte read from the network

socket
	- xxxxx

textConverter
	- xxxxx
!

!PG3SocketReadStream class methodsFor: 'instance creation ' stamp: 'ul 2/15/2009 20:25'!
on: socket

	^self basicNew
		socket: socket;
		initialize;
		yourself! !

!PG3SocketReadStream methodsFor: 'initialize-release' stamp: 'klub 8/10/2015 12:44'!
initialize

	buffer := ByteArray new: 8192.
	readLimit := 0.
	position := 0.
	offset := 0.
	bytesToKeep := 0! !

!PG3SocketReadStream methodsFor: 'private' stamp: 'klub 4/3/2016 14:59'!
jsonParser

	jsonParser ifNil: [
		jsonParser := (Smalltalk classNamed: #Json)
			ifNil: [ self ] ].
	jsonParser == self ifTrue: [ ^nil ].
	^jsonParser! !

!PG3SocketReadStream methodsFor: 'object creation' stamp: 'ul 9/15/2009 03:16'!
newDateAndTimeYear: year month: month day: day

	^self
		newDateAndTimeYear: year
		month: month
		day: day
		seconds: 0
		nanoSeconds: 0
		offset: Duration zero! !

!PG3SocketReadStream methodsFor: 'object creation' stamp: 'ul 9/7/2015 18:17'!
newDateAndTimeYear: year month: month day: day seconds: seconds nanoSeconds: nanoSeconds offset: offset

	| p q r s |
	p := month - 14 quo: 12.
	q := year + 4800 + p * 1461 quo: 4.
	r := -12 * p + month - 2 * 367 quo: 12.
	s := (year + 4900 + p quo: 100) * 3 quo: 4.
	^DateAndTime basicNew
		setJdn: q + r - s + day - 32075
		seconds: seconds
		nano: nanoSeconds
		offset: offset;
		yourself
	! !

!PG3SocketReadStream methodsFor: 'accessing' stamp: 'klub 8/10/2015 12:52'!
next

	[ position < readLimit ] whileFalse: [ self receiveData ].
	^buffer at: (position := position + 1)! !

!PG3SocketReadStream methodsFor: 'accessing' stamp: 'ul 9/15/2009 01:35'!
next: count

	| byteArray bytesAvailable |
	byteArray := ByteArray basicNew: count.
	count < 14 ifTrue: [ 
		1 to: count do: [ :i |
			byteArray at: i put: self next ].
		^byteArray ].
	(bytesAvailable := readLimit - position) >= count ifTrue: [ 
		byteArray replaceFrom: 1 to: count with: buffer startingAt: position + 1.
		position := position + count.
		^byteArray ].
	byteArray replaceFrom: 1 to: bytesAvailable with: buffer startingAt: position + 1.
	readLimit := position.
	[ bytesAvailable < count ] whileTrue: [
		bytesAvailable := bytesAvailable + (
			socket 
				waitForData;
				receiveSomeDataInto: byteArray startingAt: bytesAvailable + 1) ].
	^byteArray! !

!PG3SocketReadStream methodsFor: 'accessing' stamp: 'ul 9/6/2015 23:56'!
nextInt16

	| n |
	(n := (self next bitShift: 8) + self next) > 16r7FFF
		ifFalse: [ ^n ]
		ifTrue: [ ^n - 16r10000 ]! !

!PG3SocketReadStream methodsFor: 'accessing' stamp: 'ul 9/7/2015 17:51'!
nextInt32

	| byte result |
	(byte := self next) <= 16r7F ifTrue: [ "Is the result non-negative?"
		byte <= 16r3F ifTrue: [ "Small"
			^(((byte bitShift: 8) + self next bitShift: 8) + self next bitShift: 8) + self next ].
		"Large"
		^(LargePositiveInteger basicNew: 4)
			digitAt: 4 put: byte;
			digitAt: 3 put: self next;
			digitAt: 2 put: self next;
			digitAt: 1 put: self next;
			normalize ].
	"Negative"
	16rC0 <= byte ifTrue: [ "Small"
		^-1 - (((((byte bitShift: 8) + self next bitShift: 8) + self next bitXor: 16rFFFFFF) bitShift: 8) + (self next bitXor: 16rFF)) ].
	"Large"	
	(result := LargeNegativeInteger basicNew: 4)
		digitAt: 4 put: (byte bitXor: 16rFF);
		digitAt: 3 put: (self next bitXor: 16rFF);
		digitAt: 2 put: (self next bitXor: 16rFF).
	(byte := (self next bitXor: 16rFF) + 1) <= 16rFF ifTrue: [ "Save a LargeInteger creation and operation"
		^result
			digitAt: 1 put: byte;
			normalize ].
	^result 
		digitAt: 1 put: 16rFF;
		- 1! !

!PG3SocketReadStream methodsFor: 'accessing' stamp: 'ul 3/7/2009 05:28'!
nextInt8

	| n |
	(n := self next) > 16r7F
		ifFalse: [ ^n ]
		ifTrue: [ ^n - 16r100 ]! !

!PG3SocketReadStream methodsFor: 'accessing' stamp: 'ul 9/7/2015 00:10'!
nextString

	| endIndex result |
	((endIndex := buffer indexOf: 0 startingAt: position + 1) > 0 and: [ endIndex < readLimit ]) ifTrue: [
		(result := ByteString new: endIndex - position - 1)
			replaceFrom: 1
			to: endIndex - position - 1
			with: buffer
			startingAt: position + 1.
		position := endIndex.
		^result ].
	^String streamContents: [ :stream |
		| next |
		[ (next := self next) ~= 0 ] whileTrue: [ 
			stream nextPut: (Character value: next) ] ]! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 8/10/2015 12:24'!
parseArrayWith: aBlock length: length

	| endIndex |
	length = 2 ifTrue: [ "Empty array: '{}'"
		self next = 123 ifFalse: [ PG3SocketReadStreamError signal: '${ expected.' ].
		self next = 125 ifFalse: [ PG3SocketReadStreamError signal: '$} expected.' ].
		^#() ].
	endIndex := offset + position + length - 1.
	^Array streamContents: [ :stream |
		| byte |
		self next = 123 ifFalse: [ PG3SocketReadStreamError signal: '${ expected.' ].
		[ offset + position < endIndex ] whileTrue: [ 
			stream nextPut: aBlock value.
			((byte := self next) = 44 or: [ byte = 125 ]) ifFalse: [
				PG3SocketReadStreamError
					signal: '$, or $} expected instead of {1}({2}).'
					format:  { Character value: byte. byte } ] ].
		offset + position - 1 = endIndex ifFalse: [ 
			PG3SocketReadStreamError
				signal: 'Finished reading at {1} instead of {2}'
				format: { offset + position - 1. endIndex } ] ]! !

!PG3SocketReadStream methodsFor: 'parsing - binary format' stamp: 'ul 6/15/2017 23:53'!
parseBinaryFloat32OfLength: length

	^(FloatArray basicNew: 1)
		basicAt: 1 put: ((self next: 4) unsignedLongAt: 1 bigEndian: true);
		at: 1! !

!PG3SocketReadStream methodsFor: 'parsing - binary format' stamp: 'ul 6/15/2017 23:48'!
parseBinaryFloat64OfLength: length

	^BoxedFloat64 basicNew
		at: 1 put: ((self next: 4) unsignedLongAt: 1 bigEndian: true);
		at: 2 put: ((self next: 4) unsignedLongAt: 1 bigEndian: true);
		- 0.0! !

!PG3SocketReadStream methodsFor: 'parsing - binary format' stamp: 'ul 6/15/2017 23:36'!
parseBinaryLong64OfLength: length

	| bytes firstByte |
	bytes := self next: 8.
	firstByte := bytes at: 1.
	bytes reverseInPlace.
	LargePositiveInteger adoptInstance: bytes.
	bytes := bytes normalize.
	firstByte > 16r7F ifTrue: [ ^bytes - 16r10000000000000000 ].
	^bytes
	! !

!PG3SocketReadStream methodsFor: 'parsing - binary format' stamp: 'ul 6/14/2017 23:59'!
parseBinaryLongOfLength: length

	^self nextInt32! !

!PG3SocketReadStream methodsFor: 'parsing - binary format' stamp: 'ul 6/14/2017 23:59'!
parseBinaryShortOfLength: length

	^self nextInt16! !

!PG3SocketReadStream methodsFor: 'parsing - binary format' stamp: 'ul 6/15/2017 00:04'!
parseBinaryStringLength: length

	| result |
	result := self next: length.
	ByteString adoptInstance: result.
	^textConverter convertToSqueak: result! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'kb 9/15/2009 14:50'!
parseBoolean

	^self next = 116 " $t asciiValue "! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 8/10/2015 12:25'!
parseBooleanArrayLength: length

	^self parseArrayWith: [ self parseNull ifFalse: [ self parseBoolean ] ] length: length! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'ul 2/26/2009 12:03'!
parseBooleanLength: length

	^self next = 116 "$t asciiValue"
	
		! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 6/1/2016 17:49'!
parseByteArrayLength: length

	| writeStream i |
	writeStream := WriteStream on: (ByteArray new: length).
	i := 1.
	[ i <= length ] whileTrue: [
		| byte1 |
		(byte1 := self next) = 92 "$\ asciiValue " 
			ifTrue: [
				| byte2 |
				(byte2 := self next)
					caseOf: {
					 	[ 92 "$\ asciiValue " ] -> [ 
							writeStream nextPut: 92.
							i := i + 2 ].
						[ 120 "$x asciiValue" ] -> [
							i := i + 2.
							[ i <= length ] whileTrue: [
								writeStream nextPut: self next asCharacter digitValue * 16 + self next asCharacter digitValue.
								i := i + 2 ] ] }
					otherwise: [
						writeStream nextPut: (((byte2 - 48) * 8) + self next - 48) * 8 + self next - 48.
						i := i + 4 ] ]
			ifFalse: [
				writeStream nextPut: byte1.
				i := i + 1 ] ].
	writeStream position = length
		ifTrue: [ ^writeStream originalContents ]
		ifFalse: [ ^writeStream contents ]! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'ul 2/26/2009 16:09'!
parseCharacterLength: length

	^Character value: self next! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'ul 9/15/2009 03:20'!
parseDateLength: length

	| year month day |
	year := self parseIntegerUpTo: 45 "$- asciiValue".
	month := self parseIntegerLength: 2.
	self next.
	day := self parseIntegerLength: 2.
	year < 0 ifTrue: [ "parse ' BC'"
		self next: 3 ].
	^Date 
		starting: (self newDateAndTimeYear: year month: month day: day)	
		duration: (Duration seconds: 86400)
	! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'ul 4/15/2010 04:13'!
parseEscapedStringInArray

	^textConverter convertToSqueak: (String streamContents: [ :stream |
		| byte notEscaped |
		notEscaped := true.
		[ (byte := self next) = 34 and: [ notEscaped ] ] whileFalse: [
			notEscaped
				ifFalse: [
					stream nextPut: (Character value: byte).
					notEscaped := true ]
				ifTrue: [
					byte = 92 "$\ asciiValue"
						ifTrue: [ notEscaped := false ]
						ifFalse: [ 
							stream nextPut: (Character value: byte) ] ] ] ])! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 6/3/2016 11:14'!
parseFloat
	
	| digit i negative exponent mantissa exponent2 |
	i := 1.
	(negative := (digit := self next - 48) = -3) "$-"
		ifTrue: [ 
			digit := self next - 48.
			i := 2 ]
		ifFalse: [
			digit = 30 " $N asciiValue - 48 NaN " ifTrue: [ 
				self skip: 2.
				^Float nan ] ].
	digit = 25 " $I asciiValue - 48 [-]Infinity " ifTrue: [
		self skip: 7.
		negative ifTrue: [ ^Float infinity negated ].
		^Float infinity ].
	mantissa := digit.
	[ (digit := self next - 48) >= 0 and: [ digit <= 9 ] ] whileTrue: [
		mantissa := mantissa * 10 + digit ].
	
		digit = -2 " $. asciiValue - 48 " or: [ 
			digit =  53 " $e asciiValue - 48 " ].
	exponent := 0.
	digit = -2 "$." ifTrue: [
		[ (digit := self next - 48) >= 0 and: [ digit <= 9 ] ] whileTrue: [
			mantissa := mantissa * 10 + digit.
			exponent := exponent - 1 ] ].
	negative ifTrue: [ mantissa := mantissa negated ].
	digit = 53 "$e" ifFalse: [
		position := position - 1.
		exponent = 0 ifTrue: [ ^mantissa asFloat ].
		^(#(0.1 0.01 0.001 0.0001 1.0e-5 1.0e-6 1.0e-7 1.0e-8 1.0e-9 1.0e-10 1.0e-11 1.0e-12 1.0e-13 1.0e-14 1.0e-15 1.0e-16 1.0e-17) at: exponent negated) * mantissa asFloat ].
	negative := 44 - self next. " $- asciiValue = 45 $+ asciiValue = 43 => negative = +-1 "
	exponent2 := 0.
	[ (digit := self next - 48) >= 0 and: [ digit <= 9 ] ] whileFalse: [
		exponent2 := exponent2 * 10 + digit ].
	exponent := exponent2 * negative + exponent.
	^(10.0 raisedToInteger: exponent) * mantissa asFloat! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 8/10/2015 13:24'!
parseFloatArrayLength: length

	^self parseArrayWith: [ self parseNull ifFalse: [ self parseFloat ] ] length: length! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 8/10/2015 13:40'!
parseFloatLength: length
	
	| digit i negative exponent mantissa exponent2 |
	i := 1.
	(negative := (digit := self next - 48) = -3) "$-"
		ifTrue: [ 
			digit := self next - 48.
			i := 2 ].
	digit = 30 " $N asciiValue - 48 NaN " ifTrue: [ 
		self skip: 2.
		^Float nan ].
	digit = 25 " $I asciiValue - 48 [-]Infinity " ifTrue: [
		self skip: 7.
		negative ifTrue: [ ^Float infinity negated ].
		^Float infinity ].
	mantissa := digit.
	[ i = length or: [ 
		digit = -2 " $. asciiValue - 48 " or: [ 
			digit =  53 " $e asciiValue - 48 " ] ] ] whileFalse: [
		mantissa := mantissa * 10 + (digit := self next - 48).
		i := i + 1 ].
	i = length ifTrue: [ 
		negative ifTrue: [ ^mantissa negated asFloat ].
		^mantissa asFloat ].
	mantissa := mantissa - digit // 10.
	exponent := 0.
	digit = -2 " $. asciiValue - 48 " ifTrue: [ 
		[ i = length or: [ 
			digit = 53 " $e asciiValue - 48 " ] ] whileFalse: [
			mantissa := mantissa * 10 + (digit := self next - 48).
			exponent := exponent - 1.
			i := i + 1 ].
		digit = 53 ifTrue: [ 
			mantissa := mantissa - 53 // 10.
			exponent := exponent + 1 ] ].
	negative ifTrue: [ mantissa := mantissa negated ].
	i = length ifTrue: [
		^(#(0.1 0.01 0.001 0.0001 1.0e-5 1.0e-6 1.0e-7 1.0e-8 1.0e-9 1.0e-10 1.0e-11 1.0e-12 1.0e-13 1.0e-14 1.0e-15 1.0e-16 1.0e-17) at: exponent negated) * mantissa asFloat ].
	negative := 44 - self next. " $- asciiValue = 45 $+ asciiValue = 43 => negative = +-1 "
	i := i + 1.
	exponent2 := 0.
	[ i = length ] whileFalse: [
		exponent2 := exponent2 * 10 + (self next - 48).
		i := i + 1 ].
	exponent := exponent2 * negative + exponent.
	^(10.0 raisedTo: exponent) * mantissa asFloat! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'kb 9/15/2009 14:44'!
parseInteger

	| digit number negative |
	(negative := (number := self next - 48) = -3) "$-"
		ifTrue: [ number := 0 ].
	[ (digit := self next - 48) >= 0 and: [ digit <= 9 ] ] whileTrue: [
		number := number * 10 + digit ].
	position := position - 1. "Leave the separator on the stream."
	negative ifTrue: [ ^number negated ].
	^number! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 8/10/2015 12:25'!
parseIntegerArrayLength: length

	^self parseArrayWith: [ self parseNull ifFalse: [ self parseInteger ] ] length: length! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'ul 2/20/2009 04:01'!
parseIntegerLength: length

	| number negative |
	(negative := (number := self next - 48) = -3) "$-"
		ifTrue: [ number := 0 ].
	2 to: length do: [ :index |
		number := number * 10 + (self next - 48) ].
	negative ifTrue: [ ^number negated ].
	^number! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'ul 2/26/2009 16:21'!
parseIntegerUpTo: separatorByte

	| byte number negative |
	(negative := (number := self next - 48) = -3) "$-"
		ifTrue: [ number := 0 ].
	[ (byte := self next) = separatorByte ] whileFalse: [
		number := number * 10 + byte - 48 ].
	negative ifTrue: [ ^number negated ].
	^number! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'ul 2/26/2009 16:42'!
parseIntegerVectorLength: length

	| writeStream i |
	writeStream := WriteStream on: (Array new: length).
	i := 1.
	[ i <= length ] whileTrue: [
		| negative number byte |
		(negative := (number := self next - 48) = -3) "$-"
			ifTrue: [ number := 0 ].
		i := i + 1.
		[ i > length or: [ i := i + 1. (byte := self next) = 32 ] ] whileFalse: [
			number := number * 10 + byte - 48 ].
		writeStream nextPut: (negative 
			ifTrue: [ number negated ]
			ifFalse: [ number ]) ].
	^writeStream contents! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 3/29/2016 04:02'!
parseJsonArrayLength: length

	| json |
	json := self jsonParser ifNil: [ ^self parseStringArrayLength: length ].
	^self
		parseArrayWith: [ 
			(self parseStringInArray) ifNotNil: [ :string |
				json readFrom: string readStream ] ]
		length: length! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 3/29/2016 04:00'!
parseJsonLength: length

	| result |
	result := self parseStringLength: length.
	self jsonParser ifNotNil: [ :json |
		^json readFrom: result readStream ].
	^result! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 8/10/2015 13:15'!
parseNull
	"Return true if the next value is NULL, otherwise return false."

	| originalGlobalPosition |
	originalGlobalPosition := position + offset.
	bytesToKeep := 4.
	(self next = 78 "$N asciiValue"
		and: [ self next = 85 "$U"
		and: [ self next = 76 "$L"
		and: [ self next = 76 "$L" ] ] ])
		ifTrue: [ 
			bytesToKeep := 0.
			^true ].
	position := originalGlobalPosition -  offset.
	bytesToKeep := 0.
	^false! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 8/10/2015 13:41'!
parseScaledDecimal

	| byte i numerator negative dotPosition scale |
	(negative := (numerator := self next - 48) = -3) "$-"
		ifTrue: [ numerator := 0 ].
	numerator = 30 " $N asciiValue - 48 NaN " ifTrue: [ 
		self skip: 2.
		^Float nan ].
	i := 1.
	
	[ (byte := self next) = 44 or: [ byte = 125 ] ] whileFalse: [
		byte = 46 " $. asciiValue "
			ifTrue: [	dotPosition := i ]
			ifFalse: [ numerator := numerator * 10 + byte - 48 ].
		i := i + 1 ].
	position := position - 1.
	negative ifTrue: [ numerator := numerator negated ].
	dotPosition ifNil: [ ^numerator asScaledDecimal: 0 ].
	scale := i - dotPosition - 1.
	^ScaledDecimal 
		newFromNumber: (Fraction
			numerator: numerator
			denominator: (10 raisedToInteger: scale)) 
		scale: scale! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 8/10/2015 12:25'!
parseScaledDecimalArrayLength: length

	^self parseArrayWith: [ self parseNull ifFalse: [ self parseScaledDecimal ] ] length: length! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 8/10/2015 13:41'!
parseScaledDecimalLength: length

	| byte i numerator negative dotPosition |
	(negative := (numerator := self next - 48) = -3) "$-"
		ifTrue: [ numerator := 0 ].
	numerator = 30 " $N asciiValue - 48 NaN " ifTrue: [ 
		self skip: 2.
		^Float nan ].
	i := 1.
	[ i = length ] whileFalse: [
		(byte := self next) = 46 " $. asciiValue " ifTrue: [	
			i := i + 1.
			dotPosition := i.
			byte := self next. ].
		numerator := numerator * 10 + byte - 48.
		i := i + 1 ].
	negative ifTrue: [ numerator := numerator negated ].
	dotPosition ifNil: [ ^numerator asScaledDecimal: 0 ].
	^ScaledDecimal 
		newFromNumber: (Fraction
			numerator: numerator
			denominator: (10 raisedTo: length - dotPosition)) 
		scale: length - dotPosition
	! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'kb 9/15/2009 15:54'!
parseStringArrayLength: length

	^self parseArrayWith: [ self parseStringInArray ] length: length! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 8/9/2015 19:01'!
parseStringInArray

	| string |
	self next = 34 ifTrue: [ ^self parseEscapedStringInArray ].
	position := position - 1.
	string := String streamContents: [ :stream |
		| byte |
		[ (byte := self next) = 44 or: [ byte = 125 ] ] whileFalse: [
			stream nextPut: (Character value: byte) ] ].
	position := position - 1.
	string = 'NULL' ifTrue: [ ^nil ].
	^textConverter convertToSqueak: string! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'kb 9/15/2009 17:12'!
parseStringLength: length

	| byteString bytesAvailable |
	byteString := ByteString basicNew: length.
	length < 14 ifTrue: [ 
		1 to: length do: [ :i |
			byteString basicAt: i put: self next ].
		^textConverter convertToSqueak: byteString ].
	(bytesAvailable := readLimit - position) >= length ifTrue: [ 
		byteString replaceFrom: 1 to: length with: buffer startingAt: position + 1.
		position := position + length.
		^textConverter convertToSqueak: byteString ].
	byteString replaceFrom: 1 to: bytesAvailable with: buffer startingAt: position + 1.
	readLimit := position.
	[ bytesAvailable < length ] whileTrue: [
		bytesAvailable := bytesAvailable + (
			socket 
				waitForData;
				receiveSomeDataInto: byteString startingAt: bytesAvailable + 1) ].
	^textConverter convertToSqueak: byteString
		! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 9/3/2015 16:53'!
parseTimeArrayLength: length

	^self parseArrayWith: [ self parseNull ifFalse: [ self parseTimeLength: nil ] ] length: length! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 3/25/2015 14:12'!
parseTimeLength: length
	"Parse input having hh:mm:ss format"

	^Time
		hour: (self parseIntegerUpTo: 58 "$: asciiValue")
		minute: (self parseIntegerUpTo: 58 "$: asciiValue")
		second: (self parseIntegerLength: 2)! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 7/20/2010 22:28'!
parseTimestampWithTimezoneLength: length
	"Postgres supports several date/time formats. This method parses a subset of ISO/XSD dates used by Postgres. This means the following format: -?\d{4}-\d{2}\d{2}[ T]\d{2}:\d{2}:\d{2}(.\d+)?[\+\-]\d{2}:\d{2}:\d{2}"

	| byte |
	^self
		newDateAndTimeYear: (self parseIntegerUpTo: 45 "$- asciiValue")
		month: (self parseIntegerUpTo: 45 "$- asciiValue")
		day: (self parseIntegerLength: 2)
		seconds: 
			(self next; parseIntegerUpTo: 58 "$: asciiValue") * 60 + "hours"
			(self parseIntegerUpTo: 58 "$: asciiValue") * 60 + "minutes"
			(self parseIntegerLength: 2) "seconds"
		nanoSeconds: (
			(byte := self next) = 46 "$. asciiValue"
				ifFalse: [ 0 ]
				ifTrue: [
					| nanosecond multiplier |
					nanosecond := 0.
					multiplier := 100000000.
					[ #(43 "$+ asciiValue" 45 "$- asciiValue") includes: (byte := self next) ]
						whileFalse: [
							nanosecond := nanosecond + (byte - 48 * multiplier).
							multiplier := multiplier // 10 ].
					nanosecond ])
		offset: (Duration hours: (44 - byte) * (self parseIntegerLength: 2))! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 7/20/2010 22:28'!
parseTimestampWithoutTimezoneLength: length
	"Postgres supports several date/time formats. This method parses a subset of ISO/XSD dates used by Postgres. This means the following format: -?\d{4}-\d{2}\d{2}[ T]\d{2}:\d{2}:\d{2}(.\d+)?"

	^self
		newDateAndTimeYear: (self parseIntegerUpTo: 45 "$- asciiValue")
		month: (self parseIntegerUpTo: 45 "$- asciiValue")
		day: (self parseIntegerLength: 2)
		seconds:
			(self next; parseIntegerUpTo: 58 "$: asciiValue") * 60 + "hours"
			(self parseIntegerUpTo: 58 "$: asciiValue") * 60 + "minutes"
			(self parseIntegerLength: 2) "seconds"
		nanoSeconds: (length > 19 "has milli/microsecond part"
			ifFalse: [ 0 ]
			ifTrue: [
				(self next; parseIntegerLength: length - 20) *
				(10 raisedToInteger: 29 - length) ])
		offset: Duration zero! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 1/2/2015 11:00'!
parseVoidLength: length

	^nil
	
		! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 6/3/2014 15:56'!
parseXmlArrayLength: length

	^self parseStringArrayLength: length! !

!PG3SocketReadStream methodsFor: 'parsing - text format' stamp: 'klub 6/3/2014 15:55'!
parseXmlLength: length

	^self parseStringLength: length! !

!PG3SocketReadStream methodsFor: 'private' stamp: 'ul 9/7/2015 00:16'!
receiveData

	socket waitForData.
	bytesToKeep = 0 ifFalse: [
		bytesToKeep >= buffer size ifTrue: [
			PG3SocketReadStreamError
				signal: 'Can''t keep more than {1} bytes.'
				format: { buffer size - 1 } ]..
		buffer
			replaceFrom: 1
			to: bytesToKeep
			with: buffer
			startingAt: readLimit - bytesToKeep + 1 ].
	offset := offset + readLimit - bytesToKeep.
	readLimit := (socket receiveSomeDataInto: buffer startingAt: bytesToKeep + 1) + bytesToKeep.
	position := bytesToKeep! !

!PG3SocketReadStream methodsFor: 'accessing' stamp: 'kb 12/28/2009 17:45'!
resetOffset

	offset := 0! !

!PG3SocketReadStream methodsFor: 'accessing' stamp: 'klub 8/10/2015 13:57'!
skip: anInteger

	| remaining |
	(position := position + anInteger) <= readLimit ifTrue: [ ^self ].
	remaining := position - readLimit.
	self
		receiveData;
		skip: remaining! !

!PG3SocketReadStream methodsFor: 'accessing' stamp: 'ul 2/15/2009 20:27'!
socket: aSocket

	socket := aSocket! !

!PG3SocketReadStream methodsFor: 'accessing' stamp: 'kb 9/15/2009 17:09'!
textConverter: aPG3TextConverter

	textConverter := aPG3TextConverter! !

Object subclass: #PG3TextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3TextConverter commentStamp: 'ul 5/14/2010 21:15' prior: 0!
I convert the strings between the Postgres server and Squeak. I'm abstract, see my subclasses for details.!

PG3TextConverter subclass: #PG3NullTextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3NullTextConverter commentStamp: 'ul 5/14/2010 21:13' prior: 0!
I do the "string conversion" when the automaticStringConversion argument of a connection is false. I don't do anything just return the strings as they are.!

!PG3NullTextConverter methodsFor: 'conversion' stamp: 'ul 5/14/2010 21:12'!
convertFromSqueak: aString
	"Do nothing."

	^aString! !

!PG3NullTextConverter methodsFor: 'conversion' stamp: 'ul 5/14/2010 21:12'!
convertToSqueak: aString
	"Do nothing."

	^aString! !

PG3TextConverter subclass: #PG3SqueakTextConverter
	instanceVariableNames: 'textConverter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3SqueakTextConverter commentStamp: 'ul 5/14/2010 21:17' prior: 0!
I use Squeak's TextConverters to convert the strings between the Postgres server and Squeak.

Instance Variables
	textConverter:		<TextConverter>

textConverter
	- the text converter of Squeak I use for the conversion!

!PG3SqueakTextConverter class methodsFor: 'as yet unclassified' stamp: 'ul 3/24/2009 04:30'!
for: encodingName

	^self new
		textConverter: (TextConverter newForEncoding: encodingName);
		yourself! !

!PG3SqueakTextConverter methodsFor: 'conversion' stamp: 'ul 3/24/2009 04:18'!
convertFromSqueak: aString

	^aString convertToWithConverter: textConverter! !

!PG3SqueakTextConverter methodsFor: 'conversion' stamp: 'ul 3/24/2009 04:18'!
convertToSqueak: aString

	^aString convertFromWithConverter: textConverter! !

!PG3SqueakTextConverter methodsFor: 'accessing' stamp: 'ul 3/24/2009 04:17'!
textConverter
	^ textConverter! !

!PG3SqueakTextConverter methodsFor: 'accessing' stamp: 'ul 3/24/2009 04:17'!
textConverter: aTextConverter

	textConverter := aTextConverter! !

!PG3TextConverter methodsFor: 'conversion' stamp: 'ul 3/24/2009 04:15'!
convertFromSqueak: aString

	self subclassResponsibility! !

!PG3TextConverter methodsFor: 'conversion' stamp: 'ul 3/24/2009 04:15'!
convertToSqueak: aString

	self subclassResponsibility! !

PG3TextConverter subclass: #PG3UTF8TextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3UTF8TextConverter commentStamp: 'ul 5/14/2010 21:20' prior: 0!
I supposed to be the fast UTF-8 text converter between Postgres and Squeak. I should be benchmarked again, since Squeak's UTF8TextConverter became a lot faster in 4.1.!

!PG3UTF8TextConverter methodsFor: 'conversion' stamp: 'ul 9/17/2009 03:12'!
convertFromSqueak: aString

	1 to: aString size do: [ :indexOfNonAsciiCharacter |
		(aString at: indexOfNonAsciiCharacter) asciiValue > 127 ifTrue: [
			^self convertFromSqueak: aString from: indexOfNonAsciiCharacter ] ].
	^aString! !

!PG3UTF8TextConverter methodsFor: 'conversion' stamp: 'ul 9/7/2015 18:26'!
convertFromSqueak: aString from: indexOfNonAsciiCharacter
	
	| resultStream buffer |
	buffer := (ByteString new: aString size + 1)
		replaceFrom: 1 to: indexOfNonAsciiCharacter - 1with: aString startingAt: 1;
		yourself.
	resultStream := (WriteStream on: buffer)
		setFrom: indexOfNonAsciiCharacter to: buffer size;
		yourself.
	indexOfNonAsciiCharacter to: aString size do: [ :index |
		| character byteCount mask shift |
		(character := aString at: index) asciiValue < 128
			ifTrue: [ resultStream nextPut: character ]
			ifFalse: [ 
				| characterCode |
				characterCode := character charCode.
				byteCount := characterCode highBit + 3 // 5.
				mask := #[128 192 224 240] at: byteCount.
				shift := 1 - byteCount * 6.
				resultStream nextPut: (Character value: (characterCode bitShift: shift) + mask).
				2 to: byteCount do: [ :byteIndex |
					resultStream nextPut: (
						Character value: (
							(characterCode bitShift: (shift := shift + 6)) bitAnd: 63) + 128) ] ] ].
	resultStream position = aString size
		ifTrue: [ ^resultStream originalContents ].
	^resultStream contents! !

!PG3UTF8TextConverter methodsFor: 'conversion' stamp: 'ul 9/17/2009 03:11'!
convertToSqueak: aByteString

	1 to: aByteString size do: [ :indexOfNonAsciiCharacter |
		(aByteString at: indexOfNonAsciiCharacter) asciiValue > 127 ifTrue: [
			^self convertToSqueak: aByteString from: indexOfNonAsciiCharacter ] ].
	^aByteString! !

!PG3UTF8TextConverter methodsFor: 'conversion' stamp: 'ul 9/17/2009 03:29'!
convertToSqueak: aByteString from: indexOfNonAsciiCharacter
	
	| index resultStream buffer |
	buffer := (String new: aByteString size - 1)
		replaceFrom: 1 to: indexOfNonAsciiCharacter - 1 with: aByteString startingAt: 1;
		yourself.
	resultStream := (WriteStream on: buffer)
		setFrom: indexOfNonAsciiCharacter to: buffer size;
		yourself.
	index := indexOfNonAsciiCharacter.
	[ index <= aByteString size ] whileTrue: [ 
		| byte1 |
		(byte1 := aByteString basicAt: index) < 128
			ifTrue: [ resultStream nextPut: (aByteString at: index) ]
			ifFalse: [
				| byte2 |
				byte2 := aByteString basicAt: (index := index + 1).
				byte1 < 224
					ifTrue: [ resultStream nextPut: (
						Character value: ((byte1 bitAnd: 31) bitShift: 6) + (byte2 bitAnd: 63)) ]
					ifFalse: [
						| byte3 |
						byte3 := aByteString basicAt: (index := index + 1).
						byte1 < 240
							ifTrue: [ resultStream nextPut: (
								Character value: ((byte1 bitAnd: 15) bitShift: 12) + 
									((byte2 bitAnd: 63) bitShift: 6) + (byte3 bitAnd: 63)) ]
							ifFalse: [
								| byte4 |
								byte4 := aByteString basicAt: (index := index + 1).
								resultStream nextPut: (
									Character value: ((byte1 bitAnd: 16r7) bitShift: 18) +
										((byte2 bitAnd: 63) bitShift: 12) + 
										((byte3 bitAnd: 63) bitShift: 6) +
										(byte4 bitAnd: 63)) ] ] ].
		index := index + 1 ].
	resultStream position = aByteString size
		ifTrue: [ ^resultStream originalContents ].
	^resultStream contents! !

!Array methodsFor: '*PostgresV3-Core' stamp: 'ul 9/21/2009 00:39'!
atOrLast: anInteger

	<primitive: 60>
	^self at: self size! !

!Time methodsFor: '*PostgresV3-Core' stamp: 'ul 9/25/2014 12:51'!
pg3PrintAsLiteralOn: stream

	stream nextPut: $'.
	self print24: true showSeconds: true on: stream.
	stream nextPut: $'! !

!DateAndTime methodsFor: '*PostgresV3-Core' stamp: 'ul 9/11/2016 17:08'!
pg3PrintAsLiteralOn: stream

	stream
		nextPut: $';
		print: self;
		nextPut: $'! !

!String methodsFor: '*PostgresV3-Core' stamp: 'klub 9/12/2015 21:14'!
asPG3SqlIdentifier

	^String new: self size + 2 streamContents: [ :stream |
		self pg3PrintAsSqlIdentifierOn: stream ]! !

!String methodsFor: '*PostgresV3-Core' stamp: 'ul 9/11/2016 17:03'!
pg3PrintAsLiteralOn: stream
	"Basically String >> #storeOn:, but it has to stay the same in subclasses, like Symbol."

	| start matchIndex |
	stream nextPut: $'.
	start := 1.
	[ (matchIndex := self indexOf: $' startingAt: start) = 0 ] whileFalse: [
		stream
			next: matchIndex - start + 1 putAll: self startingAt: start;
			nextPut: $'.
		start := matchIndex + 1 ].
	stream 
		next: self size - start + 1 putAll: self startingAt: start;
		nextPut: $'! !

!String methodsFor: '*PostgresV3-Core' stamp: 'klub 9/13/2015 02:23'!
pg3PrintAsSqlIdentifierOn: stream

	| character |
	stream nextPut: $".
	1 to: self size do: [ :index |
		stream nextPut: (character := self at: index).
		(#($" $\) instVarsInclude: character) ifTrue: [
			stream nextPut: character ] ].
	stream nextPut: $"! !

!SequenceableCollection methodsFor: '*PostgresV3-Core' stamp: 'ul 9/11/2016 17:01'!
pg3PrintAsLiteralOn: stream

	self size = 0 ifTrue: [ ^stream nextPutAll: '''{}''' ].
	stream nextPutAll: 'array['.
	self 
		do: [ :each | each pg3PrintAsLiteralOn: stream ]
		separatedBy: [ stream nextPutAll: ', ' ].
	stream nextPut: $]! !

!ScaledDecimal methodsFor: '*PostgresV3-Core' stamp: 'ul 6/11/2017 21:06'!
pg3PrintAsLiteralOn: stream

	self printOn: stream showingDecimalPlaces: scale! !

!UndefinedObject methodsFor: '*PostgresV3-Core' stamp: 'ul 7/23/2010 14:29'!
pg3PrintAsLiteralOn: stream

	stream nextPutAll: 'null'! !

Error subclass: #PG3SocketReadStreamError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostgresV3-Core'!

!PG3SocketReadStreamError class methodsFor: 'instance creation' stamp: 'klub 8/10/2015 12:18'!
signal: aString format: aCollection

	^self signal: (aString format: aCollection)! !
PG3ColumnDescription initialize!
PG3AuthenticationMessage initialize!
PG3TypedMessage initialize!
